@@ -6,9 +6,9 @@
 
   Credits: Andrew Corcoran <akanewbie@gmail.com> inspired by the Alien Map GIMP plugin
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 20, 2021
-  $Id$
+  Last updated: April 23, 2023
 */
 
 #include <stdio.h>
@@ -46,7 +45,7 @@
 static const double alien_ANGLE[] = { 0, 0, 0 };
 static const double alien_FREQUENCY[] = { 1, 1, 1 };
 
-static const int alien_RADIUS = 16;
+static int alien_RADIUS = 16;
 
 enum
 {
@@ -79,7 +78,7 @@ const char *alien_descs[alien_NUM_TOOLS][2] = {
 
 // Prototypes
 Uint32 alien_api_version(void);
-int alien_init(magic_api * api);
+int alien_init(magic_api * api, Uint32 disabled_features);
 int alien_get_tool_count(magic_api * api);
 SDL_Surface *alien_get_icon(magic_api * api, int which);
 char *alien_get_name(magic_api * api, int which);
@@ -90,11 +89,16 @@ void alien_drag(magic_api * api, int which, SDL_Surface * canvas,
 Mix_Chunk *magic_loadsound(char *file);
 void alien_click(magic_api * api, int which, int mode,
                  SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void alien_release(magic_api * api, int which,
-                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void alien_release(magic_api * api, int which, SDL_Surface * canvas,
+                   SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void alien_shutdown(magic_api * api);
-void alien_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void alien_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                     SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int alien_requires_colors(magic_api * api, int which);
+Uint8 alien_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 alien_default_size(magic_api * api, int which, int mode);
+void alien_set_size(magic_api * api, int which, int mode,
+                    SDL_Surface * canvas, SDL_Surface * last, Uint8 size, SDL_Rect * update_rect);
 void alien_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void alien_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int alien_modes(magic_api * api, int which);
@@ -106,7 +110,7 @@ Uint32 alien_api_version(void)
 }
 
 //Load sounds
-int alien_init(magic_api * api)
+int alien_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   int i;
   char fname[1024];
@@ -114,10 +118,10 @@ int alien_init(magic_api * api)
   srand(time(0));
 
   for (i = 0; i < alien_NUM_TOOLS; i++)
-    {
-      snprintf(fname, sizeof(fname), "%s/sounds/magic/%s", api->data_directory, alien_snd_filenames[i]);
-      alien_snd_effect[i] = Mix_LoadWAV(fname);
-    }
+  {
+    snprintf(fname, sizeof(fname), "%ssounds/magic/%s", api->data_directory, alien_snd_filenames[i]);
+    alien_snd_effect[i] = Mix_LoadWAV(fname);
+  }
   return (1);
 }
 
@@ -164,13 +168,13 @@ static void do_alien_pixel(void *ptr, int which ATTRIBUTE_UNUSED,
 
   SDL_GetRGB(api->getpixel(canvas, x, y), canvas->format, &temp[0], &temp[1], &temp[2]);
   for (k = 0; k < 3; k++)
-    {
+  {
 //EP      temp2[k] = clamp(0,127.5 * (1.0 + sin (((temp[k] / 127.5 - 1.0) * alien_FREQUENCY[k] + alien_ANGLE[k] / 180.0) * M_PI)),255);
-      temp2[k] = clamp(0.0,
-                       127.5 * (1.0 +
-                                sin(((temp[k] / 127.5 - 1.0) * alien_FREQUENCY[k] + alien_ANGLE[k] / 180.0) * M_PI)),
-                       255.0);
-    }
+    temp2[k] = clamp(0.0,
+                     127.5 * (1.0 +
+                              sin(((temp[k] / 127.5 -
+                                    1.0) * alien_FREQUENCY[k] + alien_ANGLE[k] / 180.0) * M_PI)), 255.0);
+  }
   api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, temp2[0], temp2[1], temp2[2]));
 
 }
@@ -181,12 +185,12 @@ static void do_alien_full(void *ptr, SDL_Surface * canvas, SDL_Surface * last, i
   int x, y;
 
   for (y = 0; y < last->h; y++)
+  {
+    for (x = 0; x < last->w; x++)
     {
-      for (x = 0; x < last->w; x++)
-        {
-          do_alien_pixel(ptr, which, canvas, last, x, y);
-        }
+      do_alien_pixel(ptr, which, canvas, last, x, y);
     }
+  }
 }
 
 //do the effect for the brush
@@ -196,15 +200,15 @@ static void do_alien_brush(void *ptr, int which, SDL_Surface * canvas, SDL_Surfa
   magic_api *api = (magic_api *) ptr;
 
   for (yy = y - alien_RADIUS; yy < y + alien_RADIUS; yy++)
+  {
+    for (xx = x - alien_RADIUS; xx < x + alien_RADIUS; xx++)
     {
-      for (xx = x - alien_RADIUS; xx < x + alien_RADIUS; xx++)
-        {
-          if (api->in_circle(xx - x, yy - y, alien_RADIUS) && !api->touched(xx, yy))
-            {
-              do_alien_pixel(api, which, canvas, last, xx, yy);
-            }
-        }
+      if (api->in_circle(xx - x, yy - y, alien_RADIUS) && !api->touched(xx, yy))
+      {
+        do_alien_pixel(api, which, canvas, last, xx, yy);
+      }
     }
+  }
 }
 
 // Affect the canvas on drag:
@@ -217,19 +221,19 @@ void alien_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->playsound(alien_snd_effect[which], (x * 255) / canvas->w, 255);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
   update_rect->x = ox - alien_RADIUS;
   update_rect->y = oy - alien_RADIUS;
@@ -244,9 +248,9 @@ Mix_Chunk *magic_loadsound(char *file)
   Mix_Chunk *temp;
 
   if (!use_sound)
-    {
-      return (Mix_Chunk *) - 1;
-    }
+  {
+    return (Mix_Chunk *) - 1;
+  }
   temp = Mix_LoadWAV(file);
   return temp;
 }
@@ -258,19 +262,21 @@ void alien_click(magic_api * api, int which, int mode,
   if (mode == MODE_PAINT)
     alien_drag(api, which, canvas, last, x, y, x, y, update_rect);
   else
-    {
-      update_rect->x = 0;
-      update_rect->y = 0;
-      update_rect->w = canvas->w;
-      update_rect->h = canvas->h;
-      do_alien_full(api, canvas, last, which);
-      api->playsound(alien_snd_effect[which], 128, 255);
-    }
+  {
+    update_rect->x = 0;
+    update_rect->y = 0;
+    update_rect->w = canvas->w;
+    update_rect->h = canvas->h;
+    do_alien_full(api, canvas, last, which);
+    api->playsound(alien_snd_effect[which], 128, 255);
+  }
 }
 
 // Affect the canvas on release:
-void alien_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                   SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void alien_release(magic_api * api ATTRIBUTE_UNUSED,
+                   int which ATTRIBUTE_UNUSED,
+                   SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                   SDL_Surface * last ATTRIBUTE_UNUSED,
                    int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -282,17 +288,19 @@ void alien_shutdown(magic_api * api ATTRIBUTE_UNUSED)
   int i;
 
   for (i = 0; i < alien_NUM_TOOLS; i++)
+  {
+    if (alien_snd_effect[i] != NULL)
     {
-      if (alien_snd_effect[i] != NULL)
-        {
-          Mix_FreeChunk(alien_snd_effect[i]);
-        }
+      Mix_FreeChunk(alien_snd_effect[i]);
     }
+  }
 }
 
 // Record the color from Tux Paint:
-void alien_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
-                     Uint8 b ATTRIBUTE_UNUSED)
+void alien_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                     Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                     SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -302,13 +310,30 @@ int alien_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_
   return 0;
 }
 
-void alien_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED)
+Uint8 alien_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 6;
+}
+
+Uint8 alien_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+void alien_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                    SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                    Uint8 size, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  alien_RADIUS = size * 4;
+}
+
+void alien_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                    int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void alien_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void alien_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
