@@ -7,8 +7,8 @@
   By Bill Kendrick
   Based on `blind.c` by Pere Pujal Carabantes
 
-  Copyright (c) 2021
-  http://www.tuxpaint.org/
+  Copyright (c) 2021-2023
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
+
+  Last updated: April 18, 2023
 */
 
 #include "tp_magic_api.h"
@@ -37,8 +39,9 @@ Mix_Chunk *checkerboard_snd;
 
 // Prototypes
 Uint32 checkerboard_api_version(void);
-void checkerboard_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
-int checkerboard_init(magic_api * api);
+void checkerboard_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                            SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
+int checkerboard_init(magic_api * api, Uint32 disabled_features);
 int checkerboard_get_tool_count(magic_api * api);
 SDL_Surface *checkerboard_get_icon(magic_api * api, int which);
 char *checkerboard_get_name(magic_api * api, int which);
@@ -46,16 +49,24 @@ int checkerboard_get_group(magic_api * api, int which);
 char *checkerboard_get_description(magic_api * api, int which, int mode);
 int checkerboard_requires_colors(magic_api * api, int which);
 void checkerboard_release(magic_api * api, int which,
-                   SDL_Surface * canvas, SDL_Surface * snapshot, int x, int y, SDL_Rect * update_rect);
+                          SDL_Surface * canvas, SDL_Surface * snapshot, int x, int y, SDL_Rect * update_rect);
 void checkerboard_shutdown(magic_api * api);
-void checkerboard_paint_checkerboard(void *ptr_to_api, int which_tool, SDL_Surface * canvas, SDL_Surface * snapshot, int x, int y);
+void checkerboard_paint_checkerboard(void *ptr_to_api, int which_tool,
+                                     SDL_Surface * canvas, SDL_Surface * snapshot, int x, int y);
 void checkerboard_drag(magic_api * api, int which, SDL_Surface * canvas,
-                SDL_Surface * snapshot, int ox, int oy, int x, int y, SDL_Rect * update_rect);
+                       SDL_Surface * snapshot, int ox, int oy, int x, int y, SDL_Rect * update_rect);
 void checkerboard_click(magic_api * api, int which, int mode,
-                 SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+                        SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void checkerboard_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void checkerboard_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int checkerboard_modes(magic_api * api, int which);
+Uint8 checkerboard_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                                  int mode ATTRIBUTE_UNUSED);
+Uint8 checkerboard_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                                int mode ATTRIBUTE_UNUSED);
+void checkerboard_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                           SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                           Uint8 size ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED);
 
 //                              Housekeeping functions
 
@@ -64,18 +75,21 @@ Uint32 checkerboard_api_version(void)
   return (TP_MAGIC_API_VERSION);
 }
 
-void checkerboard_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b)       //get the colors from API and store it in structure
+void checkerboard_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                            SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r,
+                            Uint8 g, Uint8 b, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
+  //get the colors from API and store it in structure
   checkerboard_r = r;
   checkerboard_g = g;
   checkerboard_b = b;
 }
 
-int checkerboard_init(magic_api * api)
+int checkerboard_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/checkerboard.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/checkerboard.ogg", api->data_directory);
   checkerboard_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -90,7 +104,7 @@ SDL_Surface *checkerboard_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/checkerboard.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/checkerboard.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -105,11 +119,10 @@ int checkerboard_get_group(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE
   return MAGIC_TYPE_PICTURE_DECORATIONS;
 }
 
-char *checkerboard_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+char *checkerboard_get_description(magic_api * api ATTRIBUTE_UNUSED,
+                                   int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
 {
-  return
-    strdup(gettext_noop
-           ("Click and drag to fill the canvas with a checkerboard pattern."));
+  return strdup(gettext_noop("Click and drag to fill the canvas with a checkerboard pattern."));
 }
 
 int checkerboard_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
@@ -117,9 +130,11 @@ int checkerboard_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATT
   return 1;
 }
 
-void checkerboard_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                   SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * snapshot ATTRIBUTE_UNUSED,
-                   int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+void checkerboard_release(magic_api * api ATTRIBUTE_UNUSED,
+                          int which ATTRIBUTE_UNUSED,
+                          SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                          SDL_Surface * snapshot ATTRIBUTE_UNUSED,
+                          int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -130,10 +145,9 @@ void checkerboard_shutdown(magic_api * api ATTRIBUTE_UNUSED)
 
 // Interactivity functions
 
-void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
-                SDL_Surface * snapshot,
-                int ox ATTRIBUTE_UNUSED, int oy ATTRIBUTE_UNUSED,
-                int x, int y, SDL_Rect * update_rect)
+void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED,
+                       SDL_Surface * canvas, SDL_Surface * snapshot,
+                       int ox ATTRIBUTE_UNUSED, int oy ATTRIBUTE_UNUSED, int x, int y, SDL_Rect * update_rect)
 {
   int sz, xx, yy;
   Uint8 draw_start, draw_row, draw_cell;
@@ -154,11 +168,14 @@ void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface
 
   /* From the mouse Y position down... */
   draw_row = draw_start;
-  for (yy = checkerboard_start_y; yy <= canvas->h; yy += sz) {
+  for (yy = checkerboard_start_y; yy <= canvas->h; yy += sz)
+  {
     /* From the mouse X position right... */
     draw_cell = draw_row;
-    for (xx = checkerboard_start_x; xx <= canvas->w; xx += sz) {
-      if (draw_cell) {
+    for (xx = checkerboard_start_x; xx <= canvas->w; xx += sz)
+    {
+      if (draw_cell)
+      {
         dest.x = xx;
         dest.y = yy;
         dest.w = sz;
@@ -170,8 +187,10 @@ void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface
 
     /* From the mouse X position left... */
     draw_cell = !draw_row;
-    for (xx = checkerboard_start_x - sz; xx > -sz; xx -= sz) {
-      if (draw_cell) {
+    for (xx = checkerboard_start_x - sz; xx > -sz; xx -= sz)
+    {
+      if (draw_cell)
+      {
         dest.x = xx;
         dest.y = yy;
         dest.w = sz;
@@ -186,11 +205,14 @@ void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface
 
   /* From the mouse Y position up... */
   draw_row = !draw_start;
-  for (yy = checkerboard_start_y - sz; yy > -sz; yy -= sz) {
+  for (yy = checkerboard_start_y - sz; yy > -sz; yy -= sz)
+  {
     /* From the mouse X position right... */
     draw_cell = draw_row;
-    for (xx = checkerboard_start_x; xx <= canvas->w; xx += sz) {
-      if (draw_cell) {
+    for (xx = checkerboard_start_x; xx <= canvas->w; xx += sz)
+    {
+      if (draw_cell)
+      {
         dest.x = xx;
         dest.y = yy;
         dest.w = sz;
@@ -202,8 +224,10 @@ void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface
 
     /* From the mouse X position left... */
     draw_cell = !draw_row;
-    for (xx = checkerboard_start_x - sz; xx > -sz; xx -= sz) {
-      if (draw_cell) {
+    for (xx = checkerboard_start_x - sz; xx > -sz; xx -= sz)
+    {
+      if (draw_cell)
+      {
         dest.x = xx;
         dest.y = yy;
         dest.w = sz;
@@ -225,23 +249,23 @@ void checkerboard_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface
   api->playsound(checkerboard_snd, 128, 255);
 }
 
-void checkerboard_click(magic_api * api, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                 SDL_Surface * canvas, SDL_Surface * last,
-                 int x, int y,
-                 SDL_Rect * update_rect)
+void checkerboard_click(magic_api * api, int which ATTRIBUTE_UNUSED,
+                        int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas,
+                        SDL_Surface * last, int x, int y, SDL_Rect * update_rect)
 {
   checkerboard_start_x = x;
   checkerboard_start_y = y;
   checkerboard_drag(api, which, canvas, last, x, y, x, y, update_rect);
 }
 
-void checkerboard_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void checkerboard_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                           int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void checkerboard_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void checkerboard_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                            int which ATTRIBUTE_UNUSED,
+                            int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -249,3 +273,20 @@ int checkerboard_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNU
 {
   return (MODE_PAINT);
 }
+
+Uint8 checkerboard_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                                  int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+Uint8 checkerboard_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+void checkerboard_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                           SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                           Uint8 size ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+}
