@@ -1,4 +1,6 @@
-/* Last updated 2021-09-21 */
+/*
+  Last updated: April 19, 2023
+*/
 
 #include <time.h>               //For time()
 
@@ -6,9 +8,14 @@
 #include "SDL_image.h"
 #include "SDL_mixer.h"
 
-#define CONFETTI_BRUSH_SIZE 8   //radius of each confetti circle
+static int CONFETTI_BRUSH_SIZE = 8;     //radius of each confetti circle
+
 #define CONFETTI_QUANTITY 3     //how many circles will be created every click?
 
+#ifdef __ANDROID__
+#define inline static
+#endif
+
 struct confetti_rgb
 {
   Uint8 r, g, b;
@@ -20,8 +27,9 @@ Mix_Chunk *confetti_snd;
 
 /* Local function prototypes: */
 Uint32 confetti_api_version(void);
-void confetti_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
-int confetti_init(magic_api * api);
+void confetti_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                        SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
+int confetti_init(magic_api * api, Uint32 disabled_features);
 int confetti_get_tool_count(magic_api * api);
 SDL_Surface *confetti_get_icon(magic_api * api, int which);
 char *confetti_get_name(magic_api * api, int which);
@@ -44,24 +52,31 @@ int confetti_modes(magic_api * api, int which);
 
 void confetti_drag(magic_api * api, int which, SDL_Surface * canvas,
                    SDL_Surface * snapshot, int ox, int oy, int x, int y, SDL_Rect * update_rect);
+Uint8 confetti_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 confetti_default_size(magic_api * api, int which, int mode);
+void confetti_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                       SDL_Rect * update_rect);
 
 Uint32 confetti_api_version(void)
 {
   return (TP_MAGIC_API_VERSION);
 }
 
-void confetti_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b)    //get the colors from API and store it in structure
+void confetti_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                        SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g,
+                        Uint8 b, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
+  //get the colors from API and store it in structure
   confetti_colors.r = r;
   confetti_colors.g = g;
   confetti_colors.b = b;
 }
 
-int confetti_init(magic_api * api)
+int confetti_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/confetti.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/confetti.ogg", api->data_directory);
   confetti_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -76,7 +91,7 @@ SDL_Surface *confetti_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/confetti.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/confetti.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -101,8 +116,10 @@ int confetti_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBU
   return 1;
 }
 
-void confetti_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * snapshot ATTRIBUTE_UNUSED,
+void confetti_release(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                      SDL_Surface * snapshot ATTRIBUTE_UNUSED,
                       int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 
@@ -179,25 +196,25 @@ void confetti_click(magic_api * api, int which, int mode ATTRIBUTE_UNUSED,
   char dx = 0, dy = 0;
 
   for (i = 0; i < CONFETTI_QUANTITY; i++)
-    {
-      srand((dx + dy) / 2 + time(0));   //to get a unique seed even if dx and dy aren't defined
-      dx = (rand() % 100) - 50; //generate a value between <-50; +50>
-      dy = (rand() % 100) - 50; //to spread confetti around the cursor position
+  {
+    srand((dx + dy) / 2 + time(0));     //to get a unique seed even if dx and dy aren't defined
+    dx = (rand() % CONFETTI_BRUSH_SIZE * 12) - (CONFETTI_BRUSH_SIZE * 6);       //generate a value between <-50; +50>
+    dy = (rand() % CONFETTI_BRUSH_SIZE * 12) - (CONFETTI_BRUSH_SIZE * 6);       //to spread confetti around the cursor position
 
-      if (!i)
-        {
-          min_x = max_x = dx;
-          min_y = max_y = dy;
-        }
-      else
-        {
-          min_x = confetti_get_lesser(min_x, dx);       //any candidates to new min/max values? Hands up please...
-          max_x = confetti_get_greater(max_x, dx);
-          min_y = confetti_get_lesser(min_y, dy);
-          max_y = confetti_get_greater(max_y, dy);
-        }
-      confetti_circle((void *)api, which, canvas, last, x + dx, y + dy);
+    if (!i)
+    {
+      min_x = max_x = dx;
+      min_y = max_y = dy;
     }
+    else
+    {
+      min_x = confetti_get_lesser(min_x, dx);   //any candidates to new min/max values? Hands up please...
+      max_x = confetti_get_greater(max_x, dx);
+      min_y = confetti_get_lesser(min_y, dy);
+      max_y = confetti_get_greater(max_y, dy);
+    }
+    confetti_circle((void *)api, which, canvas, last, x + dx, y + dy);
+  }
 
   update_rect->x = x + min_x - CONFETTI_BRUSH_SIZE / 2;
   update_rect->y = y + min_y - CONFETTI_BRUSH_SIZE / 2;
@@ -213,28 +230,28 @@ void confetti_drag(magic_api * api, int which, SDL_Surface * canvas,
   int temp;
 
   if (ox > x)
-    {
-      temp = x;
-      x = ox;
-      ox = temp;
-    }
+  {
+    temp = x;
+    x = ox;
+    ox = temp;
+  }
   if (oy > y)
-    {
-      temp = y;
-      y = oy;
-      oy = temp;
-    }
+  {
+    temp = y;
+    y = oy;
+    oy = temp;
+  }
 
   confetti_click(api, which, MODE_PAINT, canvas, snapshot, x, y, update_rect);
 }
 
-void confetti_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                       SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void confetti_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                       int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void confetti_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                        SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void confetti_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                        int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -242,3 +259,21 @@ int confetti_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_PAINT);
 }
+
+
+Uint8 confetti_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 8;
+}
+
+Uint8 confetti_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 2;
+}
+
+void confetti_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                       SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                       Uint8 size ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  CONFETTI_BRUSH_SIZE = size * 4;
+}
