@@ -4,9 +4,9 @@
   Emboss Magic Tool Plugin
   Tux Paint - A simple drawing program for children.
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,8 +23,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: November 8, 2021
-  $Id$
+  Last updated: February 12, 2023
 */
 
 #include <stdio.h>
@@ -36,10 +35,11 @@
 /* Our globals: */
 
 static Mix_Chunk *emboss_snd;
+static int emboss_radius = 16;
 
 // Prototypes
 Uint32 emboss_api_version(void);
-int emboss_init(magic_api * api);
+int emboss_init(magic_api * api, Uint32 disabled_features);
 int emboss_get_tool_count(magic_api * api);
 SDL_Surface *emboss_get_icon(magic_api * api, int which);
 char *emboss_get_name(magic_api * api, int which);
@@ -56,11 +56,16 @@ void emboss_release(magic_api * api, int which,
                     SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 
 void emboss_shutdown(magic_api * api);
-void emboss_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void emboss_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                      SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int emboss_requires_colors(magic_api * api, int which);
 void emboss_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void emboss_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int emboss_modes(magic_api * api, int which);
+Uint8 emboss_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 emboss_default_size(magic_api * api, int which, int mode);
+void emboss_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                     SDL_Rect * update_rect);
 
 
 Uint32 emboss_api_version(void)
@@ -70,11 +75,11 @@ Uint32 emboss_api_version(void)
 
 
 // No setup required:
-int emboss_init(magic_api * api)
+int emboss_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/emboss.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/emboss.ogg", api->data_directory);
   emboss_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -91,7 +96,7 @@ SDL_Surface *emboss_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/emboss.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/emboss.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -109,15 +114,19 @@ int emboss_get_group(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSE
 }
 
 // Return our descriptions, localized:
-char *emboss_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+char *emboss_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode)
 {
-  return (strdup(gettext_noop("Click and drag the mouse to emboss the picture.")));
+  if (mode == MODE_PAINT)
+    return (strdup(gettext_noop("Click and drag the mouse to emboss the picture.")));
+  else
+    return (strdup(gettext_noop("Click to emboss the entire picture.")));
 }
 
 
 // Do the effect (single pixel; used by do_emboss() (painted circle)
 // and emboss_click() when in fullscreen mode):
-static void emboss_pixel(void * ptr, SDL_Surface * last, int x, int y, SDL_Surface * canvas) {
+static void emboss_pixel(void *ptr, SDL_Surface * last, int x, int y, SDL_Surface * canvas)
+{
   magic_api *api = (magic_api *) ptr;
   Uint8 r1, g1, b1, r2, g2, b2;
   int r;
@@ -152,19 +161,19 @@ static void do_emboss(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * canva
   magic_api *api = (magic_api *) ptr;
   int xx, yy;
 
-  for (yy = -16; yy < 16; yy++)
+  for (yy = -emboss_radius; yy < emboss_radius; yy++)
+  {
+    for (xx = -emboss_radius; xx < emboss_radius; xx++)
     {
-      for (xx = -16; xx < 16; xx++)
+      if (api->in_circle(xx, yy, emboss_radius))
+      {
+        if (!api->touched(x + xx, y + yy))
         {
-          if (api->in_circle(xx, yy, 16))
-            {
-              if (!api->touched(x + xx, y + yy))
-                {
-                  emboss_pixel(api, last, x + xx, y + yy, canvas);
-                }
-            }
+          emboss_pixel(api, last, x + xx, y + yy, canvas);
         }
+      }
     }
+  }
 }
 
 // Affect the canvas on drag:
@@ -174,24 +183,24 @@ void emboss_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->line((void *)api, which, canvas, last, ox, oy, x, y, 1, do_emboss);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
-  update_rect->x = ox - 16;
-  update_rect->y = oy - 16;
-  update_rect->w = (x + 16) - update_rect->x;
-  update_rect->h = (y + 16) - update_rect->h;
+  update_rect->x = ox - emboss_radius;
+  update_rect->y = oy - emboss_radius;
+  update_rect->w = (x + emboss_radius) - update_rect->x;
+  update_rect->h = (y + emboss_radius) - update_rect->y;
 
   api->playsound(emboss_snd, (x * 255) / canvas->w, 255);
 }
@@ -200,15 +209,21 @@ void emboss_drag(magic_api * api, int which, SDL_Surface * canvas,
 void emboss_click(magic_api * api, int which, int mode,
                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect)
 {
-  if (mode == MODE_PAINT) {
+  if (mode == MODE_PAINT)
+  {
     emboss_drag(api, which, canvas, last, x, y, x, y, update_rect);
-  } else {
-    for (y = 0; y < canvas->h; y++) {
-      if (y % 10 == 0) {
+  }
+  else
+  {
+    for (y = 0; y < canvas->h; y++)
+    {
+      if (y % 10 == 0)
+      {
         api->update_progress_bar();
       }
 
-      for (x = 0; x < canvas->w; x++) {
+      for (x = 0; x < canvas->w; x++)
+      {
         emboss_pixel(api, last, x, y, canvas);
       }
     }
@@ -221,8 +236,10 @@ void emboss_click(magic_api * api, int which, int mode,
 }
 
 // Affect the canvas on release:
-void emboss_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void emboss_release(magic_api * api ATTRIBUTE_UNUSED,
+                    int which ATTRIBUTE_UNUSED,
+                    SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                    SDL_Surface * last ATTRIBUTE_UNUSED,
                     int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -235,8 +252,10 @@ void emboss_shutdown(magic_api * api ATTRIBUTE_UNUSED)
 }
 
 // Record the color from Tux Paint:
-void emboss_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
-                      Uint8 b ATTRIBUTE_UNUSED)
+void emboss_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                      Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                      SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -246,13 +265,13 @@ int emboss_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE
   return 0;
 }
 
-void emboss_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void emboss_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void emboss_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void emboss_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -260,3 +279,24 @@ int emboss_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_PAINT | MODE_FULLSCREEN);
 }
+
+
+Uint8 emboss_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode)
+{
+  if (mode == MODE_PAINT)
+    return 8;
+  else
+    return 0;
+}
+
+Uint8 emboss_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+void emboss_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                     SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  emboss_radius = size * 4;
+}
