@@ -1,8 +1,8 @@
 /*
- * Draws fretwork
- *
- * Last updated: 2021-09-20
- */
+  Draws fretwork
+
+  Last updated: April 19, 2023
+*/
 
 #include "tp_magic_api.h"
 #include "SDL_image.h"
@@ -28,6 +28,10 @@
 #define SEG_RIGHT_TOP_BOTTOM (SEG_RIGHT | SEG_TOP | SEG_BOTTOM)
 #define SEG_LEFT_RIGHT_TOP_BOTTOM (SEG_LEFT | SEG_RIGHT | SEG_TOP | SEG_BOTTOM)
 
+#ifdef __ANDROID__
+#define inline static
+#endif
+
 Mix_Chunk *fretwork_snd;
 unsigned int img_w, img_h;
 unsigned int fretwork_segments_x, fretwork_segments_y;  //how many segments do we have?
@@ -52,9 +56,10 @@ static SDL_Surface *fretwork_one_back, *fretwork_three_back, *fretwork_four_back
 
 Uint32 fretwork_api_version(void);
 int fretwork_modes(magic_api * api, int which);
-void fretwork_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void fretwork_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                        SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 static void fretwork_colorize(magic_api * api, SDL_Surface * dest, SDL_Surface * src);
-int fretwork_init(magic_api * api);
+int fretwork_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED);
 int fretwork_get_tool_count(magic_api * api);
 SDL_Surface *fretwork_get_icon(magic_api * api, int which);
 char *fretwork_get_name(magic_api * api, int which);
@@ -66,7 +71,9 @@ void fretwork_release(magic_api * api, int which,
 void fretwork_shutdown(magic_api * api);
 void fretwork_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * snapshot);
 void fretwork_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * snapshot);
-inline void fretwork_extract_coords_from_segment(unsigned int segment, Sint16 * x, Sint16 * y);
+
+#define POINT_TYPE typeof(((SDL_Rect *)NULL)->x)
+inline void fretwork_extract_coords_from_segment(unsigned int segment, POINT_TYPE * x, POINT_TYPE * y);
 void fretwork_click(magic_api * api, int which, int mode,
                     SDL_Surface * canvas, SDL_Surface * snapshot, int x, int y, SDL_Rect * update_rect);
 
@@ -76,6 +83,10 @@ void fretwork_drag(magic_api * api, int which, SDL_Surface * canvas,
                    SDL_Surface * snapshot, int ox, int oy, int x, int y, SDL_Rect * update_rect);
 static void fretwork_draw_wrapper(void *ptr, int which, SDL_Surface * canvas, SDL_Surface * last, int x, int y);
 inline unsigned int fretwork_get_segment(int x, int y);
+Uint8 fretwork_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 fretwork_default_size(magic_api * api, int which, int mode);
+void fretwork_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                       SDL_Rect * update_rect);
 
 
 SDL_Surface *fretwork_one, *fretwork_three, *fretwork_four, *fretwork_corner;
@@ -90,7 +101,9 @@ int fretwork_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
   return (MODE_PAINT | MODE_FULLSCREEN);
 }
 
-void fretwork_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b)
+void fretwork_set_color(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                        SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b,
+                        SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
   fretwork_r = r;
   fretwork_g = g;
@@ -111,21 +124,21 @@ static void fretwork_colorize(magic_api * api, SDL_Surface * dest, SDL_Surface *
   SDL_LockSurface(dest);
 
   for (y = 0; y < src->h; y++)
+  {
+    for (x = 0; x < src->w; x++)
     {
-      for (x = 0; x < src->w; x++)
-        {
-          SDL_GetRGBA(api->getpixel(src, x, y), src->format, &r, &g, &b, &a);
+      SDL_GetRGBA(api->getpixel(src, x, y), src->format, &r, &g, &b, &a);
 
-          api->putpixel(dest, x, y, SDL_MapRGBA(dest->format, fretwork_r, fretwork_g, fretwork_b, a));
-        }
+      api->putpixel(dest, x, y, SDL_MapRGBA(dest->format, fretwork_r, fretwork_g, fretwork_b, a));
     }
+  }
 
   SDL_UnlockSurface(src);
   SDL_UnlockSurface(dest);
 }
 
 
-int fretwork_init(magic_api * api)
+int fretwork_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
   Uint8 i;                      //is always < 4, so Uint8 seems to be a good idea
@@ -135,10 +148,10 @@ int fretwork_init(magic_api * api)
   for (i = 0; i < 4; i++)
     fretwork_images[i] = (char *)malloc(sizeof(char) * 1024);
 
-  snprintf(fretwork_images[0], 1024 * sizeof(char), "%s/images/magic/fretwork_one.png", api->data_directory);
-  snprintf(fretwork_images[1], 1024 * sizeof(char), "%s/images/magic/fretwork_three.png", api->data_directory);
-  snprintf(fretwork_images[2], 1024 * sizeof(char), "%s/images/magic/fretwork_four.png", api->data_directory);
-  snprintf(fretwork_images[3], 1024 * sizeof(char), "%s/images/magic/fretwork_corner.png", api->data_directory);
+  snprintf(fretwork_images[0], 1024 * sizeof(char), "%simages/magic/fretwork_one.png", api->data_directory);
+  snprintf(fretwork_images[1], 1024 * sizeof(char), "%simages/magic/fretwork_three.png", api->data_directory);
+  snprintf(fretwork_images[2], 1024 * sizeof(char), "%simages/magic/fretwork_four.png", api->data_directory);
+  snprintf(fretwork_images[3], 1024 * sizeof(char), "%simages/magic/fretwork_corner.png", api->data_directory);
 
   fretwork_one = IMG_Load(fretwork_images[0]);
   fretwork_three = IMG_Load(fretwork_images[1]);
@@ -152,7 +165,7 @@ int fretwork_init(magic_api * api)
   img_w = fretwork_one->w;
   img_h = fretwork_one->h;
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/fretwork.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/fretwork.ogg", api->data_directory);
   fretwork_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -167,7 +180,7 @@ SDL_Surface *fretwork_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/fretwork.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/fretwork.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -195,8 +208,10 @@ int fretwork_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBU
   return 1;
 }
 
-void fretwork_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * snapshot ATTRIBUTE_UNUSED,
+void fretwork_release(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                      SDL_Surface * snapshot ATTRIBUTE_UNUSED,
                       int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -224,14 +239,16 @@ void fretwork_shutdown(magic_api * api ATTRIBUTE_UNUSED)
     free(fretwork_status_of_segments);
 }
 
-void fretwork_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+void fretwork_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                       int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
                        SDL_Surface * canvas, SDL_Surface * snapshot ATTRIBUTE_UNUSED)
 {
   //we've to compute the quantity of segments in each direction
 
-  canvas_backup = SDL_CreateRGBSurface(SDL_ANYFORMAT, canvas->w, canvas->h, canvas->format->BitsPerPixel,
-                                       canvas->format->Rmask, canvas->format->Gmask, canvas->format->Bmask,
-                                       canvas->format->Amask);
+  canvas_backup =
+    SDL_CreateRGBSurface(SDL_SWSURFACE, canvas->w, canvas->h,
+                         canvas->format->BitsPerPixel, canvas->format->Rmask,
+                         canvas->format->Gmask, canvas->format->Bmask, canvas->format->Amask);
 
   SDL_BlitSurface(canvas, NULL, canvas_backup, NULL);
   fretwork_segments_x = fretwork_math_ceil(canvas->w, img_w);
@@ -241,7 +258,8 @@ void fretwork_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNU
 }
 
 
-void fretwork_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+void fretwork_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                        int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
                         SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * snapshot ATTRIBUTE_UNUSED)
 {
   free(fretwork_status_of_segments);
@@ -274,7 +292,7 @@ inline unsigned int fretwork_get_segment(int x, int y)
   return (yy - 1) * fretwork_segments_x + xx;
 }
 
-inline void fretwork_extract_coords_from_segment(unsigned int segment, Sint16 * x, Sint16 * y)
+inline void fretwork_extract_coords_from_segment(unsigned int segment, POINT_TYPE * x, POINT_TYPE * y)
 {
   *x = ((segment % fretwork_segments_x) - 1) * img_w;   //useful to set update_rect as small as possible
   *y = (int)(segment / fretwork_segments_x) * img_h;
@@ -284,7 +302,7 @@ inline void fretwork_extract_coords_from_segment(unsigned int segment, Sint16 *
 /* { */
 /*   magic_api * api = (magic_api *) ptr; */
 
-/*   Sint16 x, y; */
+/*   POINT_TYPE x, y; */
 
 /*   for (x=0; x<dest->w; x++) */
 /*     for (y=0; y<dest->h; y++) */
@@ -294,7 +312,7 @@ inline void fretwork_extract_coords_from_segment(unsigned int segment, Sint16 *
 static void fretwork_flip_flop(void *ptr, SDL_Surface * dest, SDL_Surface * src)
 {
   magic_api *api = (magic_api *) ptr;
-  Sint16 x, y;
+  POINT_TYPE x, y;
 
   for (x = 0; x < dest->w; x++)
     for (y = 0; y < dest->h; y++)
@@ -305,20 +323,20 @@ static void fretwork_rotate(void *ptr, SDL_Surface * dest, SDL_Surface * src, _B
      //src and dest must have same size
 {
   magic_api *api = (magic_api *) ptr;
-  Sint16 x, y;
+  POINT_TYPE x, y;
 
   if (direction)                //rotate -90 degs
-    {
-      for (x = 0; x < dest->w; x++)
-        for (y = 0; y < dest->h; y++)
-          api->putpixel(dest, x, y, api->getpixel(src, y, src->h - 1 - x));
-    }
+  {
+    for (x = 0; x < dest->w; x++)
+      for (y = 0; y < dest->h; y++)
+        api->putpixel(dest, x, y, api->getpixel(src, y, src->h - 1 - x));
+  }
   else                          //rotate +90 degs
-    {
-      for (x = 0; x < dest->w; x++)
-        for (y = 0; y < dest->h; y++)
-          api->putpixel(dest, x, y, api->getpixel(src, src->h - y - 1, x));
-    }
+  {
+    for (x = 0; x < dest->w; x++)
+      for (y = 0; y < dest->h; y++)
+        api->putpixel(dest, x, y, api->getpixel(src, src->h - y - 1, x));
+  }
 
 }
 
@@ -330,42 +348,41 @@ void fretwork_click(magic_api * api, int which, int mode,
 
   fretwork_segment_modified_last = 0;
   if (mode == MODE_PAINT)
-    {
-      fretwork_segment_last_clicked = fretwork_get_segment(x, y);
-      fretwork_drag(api, which, canvas, snapshot, x, y, x, y, update_rect);
-    }
+  {
+    fretwork_segment_last_clicked = fretwork_get_segment(x, y);
+    fretwork_drag(api, which, canvas, snapshot, x, y, x, y, update_rect);
+  }
   else
+  {
+    if (fretwork_full_runs <= min(fretwork_segments_x, fretwork_segments_y) / 2)
     {
-      if (fretwork_full_runs <= min(fretwork_segments_x, fretwork_segments_y) / 2)
-        {
-          left_x = img_w * fretwork_full_runs;
-          right_x = img_w * fretwork_segments_x - img_w * fretwork_full_runs;
-          top_y = img_h * fretwork_full_runs;
-          bottom_y = img_h * fretwork_segments_y - img_h * (fretwork_full_runs - 1);
+      left_x = img_w * fretwork_full_runs;
+      right_x = img_w * fretwork_segments_x - img_w * fretwork_full_runs;
+      top_y = img_h * fretwork_full_runs;
+      bottom_y = img_h * fretwork_segments_y - img_h * (fretwork_full_runs - 1);
 
-          //left line
-          api->line((void *)api, which, canvas, snapshot, left_x, top_y, left_x, bottom_y, img_w / 2,
-                    fretwork_draw_wrapper);
+      //left line
+      api->line((void *)api, which, canvas, snapshot, left_x, top_y, left_x,
+                bottom_y, img_w / 2, fretwork_draw_wrapper);
 
-          //top line
-          api->line((void *)api, which, canvas, snapshot, left_x, top_y, right_x, top_y, img_w / 2,
-                    fretwork_draw_wrapper);
+      //top line
+      api->line((void *)api, which, canvas, snapshot, left_x, top_y, right_x, top_y, img_w / 2, fretwork_draw_wrapper);
 
-          //bottom line
-          api->line((void *)api, which, canvas, snapshot, left_x, bottom_y, right_x, bottom_y, img_w / 2,
-                    fretwork_draw_wrapper);
+      //bottom line
+      api->line((void *)api, which, canvas, snapshot, left_x, bottom_y,
+                right_x, bottom_y, img_w / 2, fretwork_draw_wrapper);
 
-          //right line
-          api->line((void *)api, which, canvas, snapshot, right_x, top_y, right_x, bottom_y, img_w / 2,
-                    fretwork_draw_wrapper);
+      //right line
+      api->line((void *)api, which, canvas, snapshot, right_x, top_y,
+                right_x, bottom_y, img_w / 2, fretwork_draw_wrapper);
 
-          fretwork_full_runs += 1;
-          update_rect->x = 0;
-          update_rect->y = 0;
-          update_rect->w = canvas->w;
-          update_rect->h = canvas->h;
-        }
+      fretwork_full_runs += 1;
+      update_rect->x = 0;
+      update_rect->y = 0;
+      update_rect->w = canvas->w;
+      update_rect->h = canvas->h;
     }
+  }
 }
 
 static Uint8 fretwork_select_image(Uint16 segment)
@@ -378,39 +395,39 @@ static Uint8 fretwork_select_image(Uint16 segment)
 
   //Checking from were we come...
   if (fretwork_segment_modified_last > 0)
-    {
-      if (segment == fretwork_segment_modified_last + 1)
-        from_left = 1;
-      else if (segment == fretwork_segment_modified_last - 1)
-        from_right = 1;
-      else if (segment == fretwork_segment_modified_last - fretwork_segments_x)
-        from_bottom = 1;
-      else if (segment == fretwork_segment_modified_last + fretwork_segments_x)
-        from_top = 1;
+  {
+    if (segment == fretwork_segment_modified_last + 1)
+      from_left = 1;
+    else if (segment == fretwork_segment_modified_last - 1)
+      from_right = 1;
+    else if (segment == fretwork_segment_modified_last - fretwork_segments_x)
+      from_bottom = 1;
+    else if (segment == fretwork_segment_modified_last + fretwork_segments_x)
+      from_top = 1;
 
-      // Very very few cases will reach this, segments are joining by the corner
-      // We need to add a new segment to join by side, adding clockwise
-      else if (segment == fretwork_segment_modified_last + fretwork_segments_x + 1)
-        {
-          from_top_left = 1;
-          fretwork_segment_to_add = segment - fretwork_segments_x;
-        }
-      else if (segment == fretwork_segment_modified_last + fretwork_segments_x - 1)
-        {
-          from_top_right = 1;
-          fretwork_segment_to_add = segment + 1;
-        }
-      else if (segment == fretwork_segment_modified_last - fretwork_segments_x - 1)
-        {
-          from_bottom_right = 1;
-          fretwork_segment_to_add = segment + fretwork_segments_x;
-        }
-      else if (segment == fretwork_segment_modified_last - fretwork_segments_x + 1)
-        {
-          from_bottom_left = 1;
-          fretwork_segment_to_add = segment - 1;
-        }
+    // Very very few cases will reach this, segments are joining by the corner
+    // We need to add a new segment to join by side, adding clockwise
+    else if (segment == fretwork_segment_modified_last + fretwork_segments_x + 1)
+    {
+      from_top_left = 1;
+      fretwork_segment_to_add = segment - fretwork_segments_x;
     }
+    else if (segment == fretwork_segment_modified_last + fretwork_segments_x - 1)
+    {
+      from_top_right = 1;
+      fretwork_segment_to_add = segment + 1;
+    }
+    else if (segment == fretwork_segment_modified_last - fretwork_segments_x - 1)
+    {
+      from_bottom_right = 1;
+      fretwork_segment_to_add = segment + fretwork_segments_x;
+    }
+    else if (segment == fretwork_segment_modified_last - fretwork_segments_x + 1)
+    {
+      from_bottom_left = 1;
+      fretwork_segment_to_add = segment - 1;
+    }
+  }
 
   take_up = segment - fretwork_segments_x;
   if (take_up <= 0)
@@ -435,9 +452,9 @@ static Uint8 fretwork_select_image(Uint16 segment)
     val_right = fretwork_status_of_segments[segment + 1];
 
   if (from_left || (val_left & SEG_RIGHT) || from_bottom_left)
-    {
-      LEFT = 1;
-    }
+  {
+    LEFT = 1;
+  }
   if (from_right || (val_right & SEG_LEFT) || from_top_right)
     RIGHT = 1;
   if (from_top || (val_up & SEG_BOTTOM) || from_top_left)
@@ -475,7 +492,8 @@ static Uint8 fretwork_select_image(Uint16 segment)
 }
 
 
-static void fretwork_draw(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
+static void fretwork_draw(void *ptr, int which ATTRIBUTE_UNUSED,
+                          SDL_Surface * canvas,
                           SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y ATTRIBUTE_UNUSED, unsigned int segment)
 {
   magic_api *api = (magic_api *) ptr;
@@ -497,73 +515,77 @@ static void fretwork_draw(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * c
 
   fretwork_status_of_segments[segment] = image; //and write it to global table
 
-  result = SDL_CreateRGBSurface(SDL_ANYFORMAT, img_w, img_h, fretwork_one->format->BitsPerPixel,
-                                fretwork_one->format->Rmask, fretwork_one->format->Gmask, fretwork_one->format->Bmask,
-                                fretwork_one->format->Amask);
+  result =
+    SDL_CreateRGBSurface(SDL_SWSURFACE, img_w, img_h,
+                         fretwork_one->format->BitsPerPixel,
+                         fretwork_one->format->Rmask,
+                         fretwork_one->format->Gmask, fretwork_one->format->Bmask, fretwork_one->format->Amask);
 
-  temp = SDL_CreateRGBSurface(SDL_ANYFORMAT, img_w, img_h, fretwork_one->format->BitsPerPixel,
-                              fretwork_one->format->Rmask, fretwork_one->format->Gmask, fretwork_one->format->Bmask,
-                              fretwork_one->format->Amask);
+  temp =
+    SDL_CreateRGBSurface(SDL_SWSURFACE, img_w, img_h,
+                         fretwork_one->format->BitsPerPixel,
+                         fretwork_one->format->Rmask,
+                         fretwork_one->format->Gmask, fretwork_one->format->Bmask, fretwork_one->format->Amask);
 
   SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
 
   switch (image)
-    {
-    case 0:
-    case SEG_TOP_BOTTOM:
-      SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
-      SDL_BlitSurface(fretwork_one, NULL, result, NULL);
-      break;
+  {
+  case 0:
+  case SEG_TOP_BOTTOM:
+    SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
+    SDL_BlitSurface(fretwork_one, NULL, result, NULL);
+    break;
 
-    case SEG_LEFT_RIGHT:
-      SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
-      fretwork_rotate(api, temp, fretwork_one, 1);
-      use_temp = 1;
-      break;
+  case SEG_LEFT_RIGHT:
+    SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
+    fretwork_rotate(api, temp, fretwork_one, 1);
+    use_temp = 1;
+    break;
 
-    case SEG_LEFT_RIGHT_TOP_BOTTOM:
-      SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
-      SDL_BlitSurface(fretwork_four, NULL, result, NULL);
-      break;
+  case SEG_LEFT_RIGHT_TOP_BOTTOM:
+    SDL_BlitSurface(canvas_backup, &modification_rect, result, NULL);
+    SDL_BlitSurface(fretwork_four, NULL, result, NULL);
+    break;
 
-    case SEG_LEFT_RIGHT_TOP:
-      SDL_BlitSurface(fretwork_three, NULL, result, NULL);
-      break;
+  case SEG_LEFT_RIGHT_TOP:
+    SDL_BlitSurface(fretwork_three, NULL, result, NULL);
+    break;
 
-    case SEG_LEFT_RIGHT_BOTTOM:
-      fretwork_flip_flop(api, temp, fretwork_three);
-      use_temp = 1;
-      break;
+  case SEG_LEFT_RIGHT_BOTTOM:
+    fretwork_flip_flop(api, temp, fretwork_three);
+    use_temp = 1;
+    break;
 
-    case SEG_LEFT_TOP_BOTTOM:
-      fretwork_rotate(api, temp, fretwork_three, 0);
-      use_temp = 1;
-      break;
+  case SEG_LEFT_TOP_BOTTOM:
+    fretwork_rotate(api, temp, fretwork_three, 0);
+    use_temp = 1;
+    break;
 
-    case SEG_RIGHT_TOP_BOTTOM:
-      fretwork_rotate(api, temp, fretwork_three, 1);
-      use_temp = 1;
-      break;
+  case SEG_RIGHT_TOP_BOTTOM:
+    fretwork_rotate(api, temp, fretwork_three, 1);
+    use_temp = 1;
+    break;
 
-    case SEG_RIGHT_TOP:
-      SDL_BlitSurface(fretwork_corner, NULL, result, NULL);
-      break;
+  case SEG_RIGHT_TOP:
+    SDL_BlitSurface(fretwork_corner, NULL, result, NULL);
+    break;
 
-    case SEG_RIGHT_BOTTOM:
-      fretwork_rotate(api, temp, fretwork_corner, 1);
-      use_temp = 1;
-      break;
+  case SEG_RIGHT_BOTTOM:
+    fretwork_rotate(api, temp, fretwork_corner, 1);
+    use_temp = 1;
+    break;
 
-    case SEG_LEFT_TOP:
-      fretwork_rotate(api, temp, fretwork_corner, 0);
-      use_temp = 1;
-      break;
+  case SEG_LEFT_TOP:
+    fretwork_rotate(api, temp, fretwork_corner, 0);
+    use_temp = 1;
+    break;
 
-    case SEG_LEFT_BOTTOM:
-      fretwork_flip_flop(api, temp, fretwork_corner);
-      use_temp = 1;
-      break;
-    }
+  case SEG_LEFT_BOTTOM:
+    fretwork_flip_flop(api, temp, fretwork_corner);
+    use_temp = 1;
+    break;
+  }
 
   if (use_temp)
     SDL_BlitSurface(temp, NULL, result, NULL);
@@ -583,20 +605,19 @@ static void fretwork_draw_wrapper(void *ptr, int which, SDL_Surface * canvas, SD
 
   if (fretwork_segment_modified_last > 0)
 
-    {
-      fretwork_draw((void *)ptr, which, canvas, last, x, y, fretwork_segment_modified_last);
-      fretwork_extract_coords_from_segment(fretwork_segment_start_rectangle, &modification_rect.x,
-                                           &modification_rect.y);
-      modification_rect.w = fretwork_update_rectangle_width * img_w;
-      modification_rect.h = fretwork_update_rectangle_height * img_h;
-    }
+  {
+    fretwork_draw((void *)ptr, which, canvas, last, x, y, fretwork_segment_modified_last);
+    fretwork_extract_coords_from_segment(fretwork_segment_start_rectangle, &modification_rect.x, &modification_rect.y);
+    modification_rect.w = fretwork_update_rectangle_width * img_w;
+    modification_rect.h = fretwork_update_rectangle_height * img_h;
+  }
 
   if (fretwork_segment_to_add > 0)
-    {
-      fretwork_draw((void *)ptr, which, canvas, last, x, y, fretwork_segment_to_add);
-      fretwork_draw((void *)ptr, which, canvas, last, x, y, fretwork_segment_modified_last);
-      fretwork_segment_to_add = 0;
-    }
+  {
+    fretwork_draw((void *)ptr, which, canvas, last, x, y, fretwork_segment_to_add);
+    fretwork_draw((void *)ptr, which, canvas, last, x, y, fretwork_segment_modified_last);
+    fretwork_segment_to_add = 0;
+  }
 
   fretwork_segment_modified_last = fretwork_segment_modified;
 }
@@ -606,27 +627,42 @@ void fretwork_drag(magic_api * api, int which,
 {
   int start_x, end_x, start_y, end_y, segment_start, segment_end, w, h;
 
-  if ((x < canvas->w) && (y < canvas->h) && (ox < canvas->w) && (oy < canvas->h) && ((signed)x > 0) && ((signed)y > 0)
-      && ((signed)ox > 0) && ((signed)oy > 0))
-    {
-      api->line((void *)api, which, canvas, snapshot, ox, oy, x, y, img_w / 2, fretwork_draw_wrapper);
-      // This should be improved, maybe passed to fretwork_draw()
-      start_x = min(ox, x);
-      end_x = max(ox, x);
-      start_y = min(oy, y);
-      end_y = max(oy, y);
-      segment_start = fretwork_get_segment(start_x - img_w, start_y - img_h);
-      segment_end = fretwork_get_segment(end_x + img_w, end_y + img_h);
+  if ((x < canvas->w) && (y < canvas->h) && (ox < canvas->w)
+      && (oy < canvas->h) && ((signed)x > 0) && ((signed)y > 0) && ((signed)ox > 0) && ((signed)oy > 0))
+  {
+    api->line((void *)api, which, canvas, snapshot, ox, oy, x, y, img_w / 2, fretwork_draw_wrapper);
+    // This should be improved, maybe passed to fretwork_draw()
+    start_x = min(ox, x);
+    end_x = max(ox, x);
+    start_y = min(oy, y);
+    end_y = max(oy, y);
+    segment_start = fretwork_get_segment(start_x - img_w, start_y - img_h);
+    segment_end = fretwork_get_segment(end_x + img_w, end_y + img_h);
 
-      x = ((segment_start % fretwork_segments_x) - 1) * img_w;
-      y = (int)(segment_start / fretwork_segments_x) * img_h;
-      w = ((segment_end % fretwork_segments_x) - 1) * img_w - x + img_w;
-      h = (int)(segment_end / fretwork_segments_x) * img_h - y + img_h;
-
-      update_rect->x = x;
-      update_rect->y = y;
-      update_rect->w = w;
-      update_rect->h = h;
-    }
+    x = ((segment_start % fretwork_segments_x) - 1) * img_w;
+    y = (int)(segment_start / fretwork_segments_x) * img_h;
+    w = ((segment_end % fretwork_segments_x) - 1) * img_w - x + img_w;
+    h = (int)(segment_end / fretwork_segments_x) * img_h - y + img_h;
 
+    update_rect->x = x;
+    update_rect->y = y;
+    update_rect->w = w;
+    update_rect->h = h;
+  }
+}
+
+Uint8 fretwork_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+Uint8 fretwork_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+void fretwork_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                       SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                       Uint8 size ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
 }
