@@ -4,9 +4,9 @@
   Metal Paint Magic Tool Plugin
   Tux Paint - A simple drawing program for children.
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,8 +23,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: April 20, 2023
 */
 
 #include <stdio.h>
@@ -37,9 +36,10 @@
 
 static Mix_Chunk *metalpaint_snd;
 static Uint8 metalpaint_r, metalpaint_g, metalpaint_b;
+static int metalpaint_size = 8;
 
 Uint32 metalpaint_api_version(void);
-int metalpaint_init(magic_api * api);
+int metalpaint_init(magic_api * api, Uint32 disabled_features);
 int metalpaint_get_tool_count(magic_api * api);
 SDL_Surface *metalpaint_get_icon(magic_api * api, int which);
 char *metalpaint_get_name(magic_api * api, int which);
@@ -50,14 +50,19 @@ void metalpaint_drag(magic_api * api, int which, SDL_Surface * canvas,
                      SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
 void metalpaint_click(magic_api * api, int which, int mode,
                       SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void metalpaint_release(magic_api * api, int which,
-                        SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void metalpaint_release(magic_api * api, int which, SDL_Surface * canvas,
+                        SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void metalpaint_shutdown(magic_api * api);
-void metalpaint_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void metalpaint_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                          SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int metalpaint_requires_colors(magic_api * api, int which);
 void metalpaint_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void metalpaint_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int metalpaint_modes(magic_api * api, int which);
+Uint8 metalpaint_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 metalpaint_default_size(magic_api * api, int which, int mode);
+void metalpaint_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                         SDL_Rect * update_rect);
 
 
 Uint32 metalpaint_api_version(void)
@@ -67,11 +72,11 @@ Uint32 metalpaint_api_version(void)
 
 
 // No setup required:
-int metalpaint_init(magic_api * api)
+int metalpaint_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/metalpaint.wav", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/metalpaint.wav", api->data_directory);
   metalpaint_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -88,7 +93,7 @@ SDL_Surface *metalpaint_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/metalpaint.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/metalpaint.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -106,8 +111,8 @@ int metalpaint_get_group(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_U
 }
 
 // Return our descriptions, localized:
-char *metalpaint_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                                 int mode ATTRIBUTE_UNUSED)
+char *metalpaint_get_description(magic_api * api ATTRIBUTE_UNUSED,
+                                 int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
 {
   return (strdup(gettext_noop("Click and drag the mouse to paint with a metallic color.")));
 }
@@ -125,27 +130,27 @@ static int metalpaint_gradient[METALPAINT_CYCLE] = {
 
 // Do the effect:
 
-static void do_metalpaint(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
-                          SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
+static void do_metalpaint(void *ptr, int which ATTRIBUTE_UNUSED,
+                          SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
 {
   magic_api *api = (magic_api *) ptr;
   int xx, yy;
   int n;
   Uint8 r, g, b;
 
-  for (yy = -8; yy < 8; yy++)
+  for (yy = -metalpaint_size; yy < metalpaint_size; yy++)
+  {
+    for (xx = -metalpaint_size; xx < metalpaint_size; xx++)
     {
-      for (xx = -8; xx < 8; xx++)
-        {
-          n = metalpaint_gradient[((x + xx + y + yy) / 4) % METALPAINT_CYCLE];
+      n = metalpaint_gradient[((x + xx + y + yy) / 4) % METALPAINT_CYCLE];
 
-          r = (metalpaint_r * n) / 255;
-          g = (metalpaint_g * n) / 255;
-          b = (metalpaint_b * n) / 255;
+      r = (metalpaint_r * n) / 255;
+      g = (metalpaint_g * n) / 255;
+      b = (metalpaint_b * n) / 255;
 
-          api->putpixel(canvas, x + xx, y + yy, SDL_MapRGB(canvas->format, r, g, b));
-        }
+      api->putpixel(canvas, x + xx, y + yy, SDL_MapRGB(canvas->format, r, g, b));
     }
+  }
 }
 
 // Affect the canvas on drag:
@@ -155,24 +160,24 @@ void metalpaint_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->line((void *)api, which, canvas, last, ox, oy, x, y, 1, do_metalpaint);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
-  update_rect->x = ox - 8;
-  update_rect->y = oy - 8;
-  update_rect->w = (x + 8) - update_rect->x;
-  update_rect->h = (y + 8) - update_rect->h;
+  update_rect->x = ox - metalpaint_size;
+  update_rect->y = oy - metalpaint_size;
+  update_rect->w = (x + metalpaint_size) - update_rect->x;
+  update_rect->h = (y + metalpaint_size) - update_rect->y;
 
   api->playsound(metalpaint_snd, (x * 255) / canvas->w, 255);
 }
@@ -185,8 +190,10 @@ void metalpaint_click(magic_api * api, int which, int mode ATTRIBUTE_UNUSED,
 }
 
 // Affect the canvas on release:
-void metalpaint_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                        SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void metalpaint_release(magic_api * api ATTRIBUTE_UNUSED,
+                        int which ATTRIBUTE_UNUSED,
+                        SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                        SDL_Surface * last ATTRIBUTE_UNUSED,
                         int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -199,7 +206,9 @@ void metalpaint_shutdown(magic_api * api ATTRIBUTE_UNUSED)
 }
 
 // Record the color from Tux Paint:
-void metalpaint_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b)
+void metalpaint_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                          SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g,
+                          Uint8 b, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
   metalpaint_r = min(255, r + 64);
   metalpaint_g = min(255, g + 64);
@@ -212,13 +221,13 @@ int metalpaint_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRI
   return 1;
 }
 
-void metalpaint_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                         SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void metalpaint_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                         int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void metalpaint_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                          SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void metalpaint_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                          int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -226,3 +235,21 @@ int metalpaint_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSE
 {
   return (MODE_PAINT);
 }
+
+
+Uint8 metalpaint_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 6;
+}
+
+Uint8 metalpaint_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 2;
+}
+
+void metalpaint_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                         SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                         SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  metalpaint_size = size * 4;
+}
