@@ -7,9 +7,9 @@
 
   Credits: Andrew Corcoran <akanewbie@gmail.com>
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,8 +26,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: April 22, 2023
 */
 
 #include <stdio.h>
@@ -52,7 +51,7 @@ static void reset_mosaic_blured(SDL_Surface * canvas);
 
 /* Prototypes */
 Uint32 mosaic_api_version(void);
-int mosaic_init(magic_api *);
+int mosaic_init(magic_api *, Uint32);
 int mosaic_get_tool_count(magic_api *);
 SDL_Surface *mosaic_get_icon(magic_api *, int);
 char *mosaic_get_name(magic_api *, int);
@@ -63,14 +62,20 @@ void mosaic_drag(magic_api *, int, SDL_Surface *, SDL_Surface *, int, int, int,
 void mosaic_click(magic_api *, int, int, SDL_Surface *, SDL_Surface *, int, int, SDL_Rect *);
 void mosaic_release(magic_api *, int, SDL_Surface *, SDL_Surface *, int, int, SDL_Rect *);
 void mosaic_shutdown(magic_api *);
-void mosaic_set_color(magic_api *, Uint8, Uint8, Uint8);
+void mosaic_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                      SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int mosaic_requires_colors(magic_api *, int);
 void mosaic_switchin(magic_api *, int, int, SDL_Surface *);
 void mosaic_switchout(magic_api *, int, int, SDL_Surface *);
 int mosaic_modes(magic_api *, int);
+Uint8 mosaic_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 mosaic_default_size(magic_api * api, int which, int mode);
+void mosaic_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                     SDL_Rect * update_rect);
+
 
 static const int mosaic_AMOUNT = 300;
-static const int mosaic_RADIUS = 16;
+static int mosaic_RADIUS = 16;
 static const double mosaic_SHARPEN = 1.0;
 static int randnoise ATTRIBUTE_UNUSED;
 Uint8 *mosaic_blured;
@@ -112,17 +117,17 @@ Uint32 mosaic_api_version(void)
 }
 
 //Load sounds
-int mosaic_init(magic_api * api)
+int mosaic_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
 
   int i;
   char fname[1024];
 
   for (i = 0; i < mosaic_NUM_TOOLS; i++)
-    {
-      snprintf(fname, sizeof(fname), "%s/sounds/magic/%s", api->data_directory, mosaic_snd_filenames[i]);
-      mosaic_snd_effect[i] = Mix_LoadWAV(fname);
-    }
+  {
+    snprintf(fname, sizeof(fname), "%ssounds/magic/%s", api->data_directory, mosaic_snd_filenames[i]);
+    mosaic_snd_effect[i] = Mix_LoadWAV(fname);
+  }
 
   return (1);
 }
@@ -181,30 +186,31 @@ static void do_mosaic_full(void *ptr, SDL_Surface * canvas,
                                                   canvas->format->BitsPerPixel,
                                                   canvas->format->Rmask,
                                                   canvas->format->Gmask,
-                                                  canvas->format->Bmask, amask);
+                                                  canvas->format->Bmask,
+                                                  amask);
 
 
 
   api->update_progress_bar();
 
   for (y = 0; y < canvas->h; y++)
-    {
+  {
 
-      for (x = 0; x < canvas->w; x++)
-        {
-          mosaic_blur_pixel(api, mosaic_temp, canvas_noise, x, y);
-        }
+    for (x = 0; x < canvas->w; x++)
+    {
+      mosaic_blur_pixel(api, mosaic_temp, canvas_noise, x, y);
     }
+  }
 
   api->update_progress_bar();
 
   for (y = 0; y < canvas->h; y++)
+  {
+    for (x = 0; x < canvas->w; x++)
     {
-      for (x = 0; x < canvas->w; x++)
-        {
-          mosaic_sharpen_pixel(api, canvas, mosaic_temp, x, y);
-        }
+      mosaic_sharpen_pixel(api, canvas, mosaic_temp, x, y);
     }
+  }
   SDL_FreeSurface(mosaic_temp);
 }
 
@@ -218,24 +224,24 @@ void mosaic_paint(void *ptr_to_api, int which ATTRIBUTE_UNUSED,
   magic_api *api = (magic_api *) ptr_to_api;
 
   for (j = max(0, y - mosaic_RADIUS - 2); j < min(canvas->h, y + mosaic_RADIUS + 2); j++)
-    {
-      pix_row_pos = j * canvas->w;
-      for (i = max(0, x - mosaic_RADIUS - 2); i < min(canvas->w, x + mosaic_RADIUS + 2); i++)
-        if (!mosaic_blured[pix_row_pos + i] && api->in_circle(i - x, j - y, mosaic_RADIUS + 2))
-          {
-            mosaic_blur_pixel(api, canvas_blur, canvas_noise, i, j);
-            mosaic_blured[pix_row_pos + i] = 1; /* Track what are yet blured */
-          }
-    }
+  {
+    pix_row_pos = j * canvas->w;
+    for (i = max(0, x - mosaic_RADIUS - 2); i < min(canvas->w, x + mosaic_RADIUS + 2); i++)
+      if (!mosaic_blured[pix_row_pos + i] && api->in_circle(i - x, j - y, mosaic_RADIUS + 2))
+      {
+        mosaic_blur_pixel(api, canvas_blur, canvas_noise, i, j);
+        mosaic_blured[pix_row_pos + i] = 1;     /* Track what are yet blured */
+      }
+  }
 
   for (i = x - mosaic_RADIUS; i < x + mosaic_RADIUS; i++)
     for (j = y - mosaic_RADIUS; j < y + mosaic_RADIUS; j++)
       if (api->in_circle(i - x, j - y, mosaic_RADIUS))
         if (!api->touched(i, j))
-          {
-            mosaic_sharpen_pixel(api, canvas_sharp, canvas_blur, i, j);
-            api->putpixel(canvas, i, j, api->getpixel(canvas_sharp, i, j));
-          }
+        {
+          mosaic_sharpen_pixel(api, canvas_sharp, canvas_blur, i, j);
+          api->putpixel(canvas, i, j, api->getpixel(canvas_sharp, i, j));
+        }
 }
 
 // Affect the canvas on drag:
@@ -258,16 +264,16 @@ void mosaic_click(magic_api * api, int which, int mode,
 {
 
   if (mode == MODE_FULLSCREEN)
-    {
-      update_rect->x = 0;
-      update_rect->y = 0;
-      update_rect->w = canvas->w;
-      update_rect->h = canvas->h;
+  {
+    update_rect->x = 0;
+    update_rect->y = 0;
+    update_rect->w = canvas->w;
+    update_rect->h = canvas->h;
 
-      do_mosaic_full(api, canvas_noise, last, which);
-      SDL_BlitSurface(canvas_noise, NULL, canvas, NULL);
-      api->playsound(mosaic_snd_effect[which], 128, 255);
-    }
+    do_mosaic_full(api, canvas_noise, last, which);
+    SDL_BlitSurface(canvas_noise, NULL, canvas, NULL);
+    api->playsound(mosaic_snd_effect[which], 128, 255);
+  }
   else
     mosaic_drag(api, which, canvas, last, x, y, x, y, update_rect);
 
@@ -289,17 +295,19 @@ void mosaic_shutdown(magic_api * api ATTRIBUTE_UNUSED)
   int i;
 
   for (i = 0; i < mosaic_NUM_TOOLS; i++)
+  {
+    if (mosaic_snd_effect[i] != NULL)
     {
-      if (mosaic_snd_effect[i] != NULL)
-        {
-          Mix_FreeChunk(mosaic_snd_effect[i]);
-        }
+      Mix_FreeChunk(mosaic_snd_effect[i]);
     }
+  }
 }
 
 // Record the color from Tux Paint:
-void mosaic_set_color(magic_api * api ATTRIBUTE_UNUSED,
-                      Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED)
+void mosaic_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                      Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                      SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -320,9 +328,9 @@ static void mosaic_noise_pixel(void *ptr, SDL_Surface * canvas, int noise_AMOUNT
 
   SDL_GetRGB(api->getpixel(canvas, x, y), canvas->format, &temp[0], &temp[1], &temp[2]);
   for (k = 0; k < 3; k++)
-    {
-      temp2[k] = clamp(0.0, (int)temp[k] - (rand() % noise_AMOUNT) + noise_AMOUNT / 2.0, 255.0);
-    }
+  {
+    temp2[k] = clamp(0.0, (int)temp[k] - (rand() % noise_AMOUNT) + noise_AMOUNT / 2.0, 255.0);
+  }
   api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, temp2[0], temp2[1], temp2[2]));
 }
 
@@ -343,26 +351,26 @@ static void mosaic_blur_pixel(void *ptr, SDL_Surface * canvas, SDL_Surface * las
   };
 
   for (k = 0; k < 3; k++)
-    {
-      blurValue[k] = 0;
-    }
+  {
+    blurValue[k] = 0;
+  }
 
   for (i = -2; i < 3; i++)
+  {
+    for (j = -2; j < 3; j++)
     {
-      for (j = -2; j < 3; j++)
-        {
-          //Add the pixels around the current one wieghted 
-          SDL_GetRGB(api->getpixel(last, x + i, y + j), last->format, &temp[0], &temp[1], &temp[2]);
-          for (k = 0; k < 3; k++)
-            {
-              blurValue[k] += temp[k] * weight[i + 2][j + 2];
-            }
-        }
+      //Add the pixels around the current one wieghted 
+      SDL_GetRGB(api->getpixel(last, x + i, y + j), last->format, &temp[0], &temp[1], &temp[2]);
+      for (k = 0; k < 3; k++)
+      {
+        blurValue[k] += temp[k] * weight[i + 2][j + 2];
+      }
     }
+  }
   for (k = 0; k < 3; k++)
-    {
-      blurValue[k] /= 273;
-    }
+  {
+    blurValue[k] /= 273;
+  }
   api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, blurValue[0], blurValue[1], blurValue[2]));
 }
 
@@ -391,24 +399,26 @@ static void mosaic_sharpen_pixel(void *ptr, SDL_Surface * canvas, SDL_Surface *
   sobel_1 = 0;
   sobel_2 = 0;
   for (i = -1; i < 2; i++)
+  {
+    for (j = -1; j < 2; j++)
     {
-      for (j = -1; j < 2; j++)
-        {
-          //No need to check if inside canvas, getpixel does it for us.
-          SDL_GetRGB(api->getpixel(last, x + i, y + j), last->format, &r1, &g1, &b1);
-          grey = mosaic_grey(r1, g1, b1);
-          sobel_1 += grey * sobel_weights_1[i + 1][j + 1];
-          sobel_2 += grey * sobel_weights_2[i + 1][j + 1];
-        }
+      //No need to check if inside canvas, getpixel does it for us.
+      SDL_GetRGB(api->getpixel(last, x + i, y + j), last->format, &r1, &g1, &b1);
+      grey = mosaic_grey(r1, g1, b1);
+      sobel_1 += grey * sobel_weights_1[i + 1][j + 1];
+      sobel_2 += grey * sobel_weights_2[i + 1][j + 1];
     }
+  }
 
   temp = sqrt(sobel_1 * sobel_1 + sobel_2 * sobel_2);
   temp = (temp / 1443) * 255.0;
 
   SDL_GetRGB(api->getpixel(last, x, y), last->format, &r1, &g1, &b1);
-  api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, clamp(0.0, r1 + mosaic_SHARPEN * temp, 255.0),
-                                         clamp(0.0, g1 + mosaic_SHARPEN * temp, 255.0),
-                                         clamp(0.0, b1 + mosaic_SHARPEN * temp, 255.0)));
+  api->putpixel(canvas, x, y,
+                SDL_MapRGB(canvas->format,
+                           clamp(0.0, r1 + mosaic_SHARPEN * temp, 255.0),
+                           clamp(0.0, g1 + mosaic_SHARPEN * temp, 255.0),
+                           clamp(0.0, b1 + mosaic_SHARPEN * temp, 255.0)));
 
 }
 
@@ -419,10 +429,10 @@ void mosaic_switchin(magic_api * api, int which ATTRIBUTE_UNUSED, int mode ATTRI
 
   mosaic_blured = (Uint8 *) malloc(sizeof(Uint8) * (canvas->w * canvas->h));
   if (mosaic_blured == NULL)
-    {
-      fprintf(stderr, "\nError: Can't build drawing touch mask!\n");
-      exit(1);
-    }
+  {
+    fprintf(stderr, "\nError: Can't build drawing touch mask!\n");
+    exit(1);
+  }
 
   amask = ~(canvas->format->Rmask | canvas->format->Gmask | canvas->format->Bmask);
 
@@ -435,12 +445,12 @@ void mosaic_switchin(magic_api * api, int which ATTRIBUTE_UNUSED, int mode ATTRI
   SDL_BlitSurface(canvas, NULL, canvas_noise, NULL);
 
   for (y = 0; y < canvas->h; y++)
+  {
+    for (x = 0; x < canvas->w; x++)
     {
-      for (x = 0; x < canvas->w; x++)
-        {
-          mosaic_noise_pixel(api, canvas_noise, mosaic_AMOUNT, x, y);
-        }
+      mosaic_noise_pixel(api, canvas_noise, mosaic_AMOUNT, x, y);
     }
+  }
 
   canvas_blur = SDL_CreateRGBSurface(SDL_SWSURFACE,
                                      canvas->w,
@@ -479,3 +489,24 @@ void reset_mosaic_blured(SDL_Surface * canvas)
       mosaic_blured[j * canvas->w + i] = 0;
 
 }
+
+
+Uint8 mosaic_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode)
+{
+  if (mode == MODE_PAINT)
+    return 8;
+  else
+    return 0;
+}
+
+Uint8 mosaic_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+void mosaic_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                     SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  mosaic_RADIUS = size * 4;
+}
