@@ -6,9 +6,9 @@
 
   Credits: Andrew Corcoran <akanewbie@gmail.com>
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: April 22, 2023
 */
 
 #include <stdio.h>
@@ -44,7 +43,7 @@
 #endif
 
 static const int noise_AMOUNT = 100.0;
-static const int noise_RADIUS = 16;
+static int noise_RADIUS = 16;
 
 enum
 {
@@ -76,7 +75,7 @@ const char *noise_descs[noise_NUM_TOOLS][2] = {
 };
 
 Uint32 noise_api_version(void);
-int noise_init(magic_api * api);
+int noise_init(magic_api * api, Uint32);
 SDL_Surface *noise_get_icon(magic_api * api, int which);
 char *noise_get_name(magic_api * api, int which);
 int noise_get_group(magic_api * api, int which);
@@ -86,17 +85,23 @@ static void do_noise_full(void *ptr, SDL_Surface * canvas, SDL_Surface * last, i
 static void do_noise_brush(void *ptr, int which, SDL_Surface * canvas, SDL_Surface * last, int x, int y);
 void noise_drag(magic_api * api, int which, SDL_Surface * canvas,
                 SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
-void noise_click(magic_api * api, int which, int mode,
-                 SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void noise_release(magic_api * api, int which,
-                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void noise_click(magic_api * api, int which, int mode, SDL_Surface * canvas,
+                 SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void noise_release(magic_api * api, int which, SDL_Surface * canvas,
+                   SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void noise_shutdown(magic_api * api);
-void noise_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void noise_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                     SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int noise_requires_colors(magic_api * api, int which);
 void noise_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void noise_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int noise_modes(magic_api * api, int which);
 int noise_get_tool_count(magic_api * api ATTRIBUTE_UNUSED);
+Uint8 noise_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 noise_default_size(magic_api * api, int which, int mode);
+void noise_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                    SDL_Rect * update_rect);
+
 
 Uint32 noise_api_version(void)
 {
@@ -104,7 +109,7 @@ Uint32 noise_api_version(void)
 }
 
 //Load sounds
-int noise_init(magic_api * api)
+int noise_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   int i;
   char fname[1024];
@@ -112,10 +117,10 @@ int noise_init(magic_api * api)
   srand(time(0));
 
   for (i = 0; i < noise_NUM_TOOLS; i++)
-    {
-      snprintf(fname, sizeof(fname), "%s/sounds/magic/%s", api->data_directory, noise_snd_filenames[i]);
-      noise_snd_effect[i] = Mix_LoadWAV(fname);
-    }
+  {
+    snprintf(fname, sizeof(fname), "%ssounds/magic/%s", api->data_directory, noise_snd_filenames[i]);
+    noise_snd_effect[i] = Mix_LoadWAV(fname);
+  }
   return (1);
 }
 
@@ -163,9 +168,9 @@ static void do_noise_pixel(void *ptr, int which ATTRIBUTE_UNUSED,
 
   SDL_GetRGB(api->getpixel(canvas, x, y), canvas->format, &temp[0], &temp[1], &temp[2]);
   for (k = 0; k < 3; k++)
-    {
-      temp2[k] = clamp(0.0, (int)temp[k] - (rand() % noise_AMOUNT) + noise_AMOUNT / 2.0, 255.0);
-    }
+  {
+    temp2[k] = clamp(0.0, (int)temp[k] - (rand() % noise_AMOUNT) + noise_AMOUNT / 2.0, 255.0);
+  }
   api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, temp2[0], temp2[1], temp2[2]));
 
 }
@@ -176,12 +181,12 @@ static void do_noise_full(void *ptr, SDL_Surface * canvas, SDL_Surface * last, i
   int x, y;
 
   for (y = 0; y < last->h; y++)
+  {
+    for (x = 0; x < last->w; x++)
     {
-      for (x = 0; x < last->w; x++)
-        {
-          do_noise_pixel(ptr, which, canvas, last, x, y);
-        }
+      do_noise_pixel(ptr, which, canvas, last, x, y);
     }
+  }
 }
 
 //do the effect for the brush
@@ -191,15 +196,15 @@ static void do_noise_brush(void *ptr, int which, SDL_Surface * canvas, SDL_Surfa
   magic_api *api = (magic_api *) ptr;
 
   for (yy = y - noise_RADIUS; yy < y + noise_RADIUS; yy++)
+  {
+    for (xx = x - noise_RADIUS; xx < x + noise_RADIUS; xx++)
     {
-      for (xx = x - noise_RADIUS; xx < x + noise_RADIUS; xx++)
-        {
-          if (api->in_circle(xx - x, yy - y, noise_RADIUS) && !api->touched(xx, yy))
-            {
-              do_noise_pixel(api, which, canvas, last, xx, yy);
-            }
-        }
+      if (api->in_circle(xx - x, yy - y, noise_RADIUS) && !api->touched(xx, yy))
+      {
+        do_noise_pixel(api, which, canvas, last, xx, yy);
+      }
     }
+  }
 }
 
 // Affect the canvas on drag:
@@ -212,19 +217,19 @@ void noise_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->playsound(noise_snd_effect[which], (x * 255) / canvas->w, 255);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
   update_rect->x = ox - noise_RADIUS;
   update_rect->y = oy - noise_RADIUS;
@@ -239,19 +244,21 @@ void noise_click(magic_api * api, int which, int mode,
   if (mode == MODE_PAINT)
     noise_drag(api, which, canvas, last, x, y, x, y, update_rect);
   else
-    {
-      update_rect->x = 0;
-      update_rect->y = 0;
-      update_rect->w = canvas->w;
-      update_rect->h = canvas->h;
-      do_noise_full(api, canvas, last, which);
-      api->playsound(noise_snd_effect[which], 128, 255);
-    }
+  {
+    update_rect->x = 0;
+    update_rect->y = 0;
+    update_rect->w = canvas->w;
+    update_rect->h = canvas->h;
+    do_noise_full(api, canvas, last, which);
+    api->playsound(noise_snd_effect[which], 128, 255);
+  }
 }
 
 // Affect the canvas on release:
-void noise_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                   SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void noise_release(magic_api * api ATTRIBUTE_UNUSED,
+                   int which ATTRIBUTE_UNUSED,
+                   SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                   SDL_Surface * last ATTRIBUTE_UNUSED,
                    int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -263,17 +270,19 @@ void noise_shutdown(magic_api * api ATTRIBUTE_UNUSED)
   int i;
 
   for (i = 0; i < noise_NUM_TOOLS; i++)
+  {
+    if (noise_snd_effect[i] != NULL)
     {
-      if (noise_snd_effect[i] != NULL)
-        {
-          Mix_FreeChunk(noise_snd_effect[i]);
-        }
+      Mix_FreeChunk(noise_snd_effect[i]);
     }
+  }
 }
 
 // Record the color from Tux Paint:
-void noise_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
-                     Uint8 b ATTRIBUTE_UNUSED)
+void noise_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                     Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                     SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -283,13 +292,13 @@ int noise_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_
   return 0;
 }
 
-void noise_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void noise_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                    int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void noise_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void noise_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -297,3 +306,24 @@ int noise_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_FULLSCREEN | MODE_PAINT);
 }
+
+
+Uint8 noise_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode)
+{
+  if (mode == MODE_PAINT)
+    return 8;
+  else
+    return 0;
+}
+
+Uint8 noise_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+void noise_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                    SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                    SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  noise_RADIUS = size * 4;
+}
