@@ -4,9 +4,9 @@
   Pixel art paintbrush Magic Tools Plugin
   Tux Paint - A simple drawing program for children.
 
-  Copyright (c) 2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,8 +23,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: April 23, 2023
 */
 
 #include <stdio.h>
@@ -46,11 +45,12 @@ enum
 
 static Mix_Chunk *pixel_snd;
 static Uint8 pixels_r, pixels_g, pixels_b;
+static int pixel_size = 8;
 
 
 /* Local function prototype: */
 
-int pixels_init(magic_api * api);
+int pixels_init(magic_api * api, Uint32 disabled_features);
 Uint32 pixels_api_version(void);
 int pixels_get_tool_count(magic_api * api);
 SDL_Surface *pixels_get_icon(magic_api * api, int which);
@@ -59,22 +59,29 @@ int pixels_get_group(magic_api * api, int which);
 char *pixels_get_description(magic_api * api, int which, int mode);
 void pixels_drag(magic_api * api, int which, SDL_Surface * canvas,
                  SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
-void pixels_click(magic_api * api, int which, int mode,
-                  SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void pixels_release(magic_api * api, int which, SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void pixels_click(magic_api * api, int which, int mode, SDL_Surface * canvas,
+                  SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void pixels_release(magic_api * api, int which, SDL_Surface * canvas,
+                    SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void pixels_shutdown(magic_api * api);
-void pixels_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void pixels_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                      SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int pixels_requires_colors(magic_api * api, int which);
 void pixels_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void pixels_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int pixels_modes(magic_api * api, int which);
+Uint8 pixels_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 pixels_default_size(magic_api * api, int which, int mode);
+void pixels_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                     SDL_Rect * update_rect);
+
 
 // No setup required:
-int pixels_init(magic_api * api)
+int pixels_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/pixels.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/pixels.ogg", api->data_directory);
   pixel_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -96,7 +103,7 @@ SDL_Surface *pixels_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/pixels.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/pixels.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -123,12 +130,10 @@ char *pixels_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBU
 
 // Do the effect:
 
-static void do_pixels(void *ptr ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
+static void do_pixels(void *ptr ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
 {
   SDL_Rect dest;
-  int pixel_size;
-
-  pixel_size = 8;
 
   dest.x = (x / pixel_size) * pixel_size;
   dest.y = (y / pixel_size) * pixel_size;
@@ -145,24 +150,29 @@ void pixels_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->line((void *)api, which, canvas, last, ox, oy, x, y, 1, do_pixels);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
-  update_rect->x = x - 64;
-  update_rect->y = y - 64;
-  update_rect->w = (ox + 128) - update_rect->x;
-  update_rect->h = (oy + 128) - update_rect->h;
+  ox = (ox / pixel_size) * pixel_size;
+  oy = (oy / pixel_size) * pixel_size;
+  x = (x / pixel_size) * pixel_size;
+  y = (y / pixel_size) * pixel_size;
+
+  update_rect->x = ox - pixel_size * 2;
+  update_rect->y = oy - pixel_size * 2;
+  update_rect->w = (x + pixel_size * 2) - update_rect->x;
+  update_rect->h = (y + pixel_size * 2) - update_rect->y;
 
   api->playsound(pixel_snd, (x * 255) / canvas->w, 255);
 }
@@ -175,7 +185,8 @@ void pixels_click(magic_api * api, int which, int mode ATTRIBUTE_UNUSED,
 }
 
 void pixels_release(magic_api * api, int which ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                    SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                    SDL_Surface * last ATTRIBUTE_UNUSED,
                     int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
   api->stopsound();
@@ -189,7 +200,9 @@ void pixels_shutdown(magic_api * api ATTRIBUTE_UNUSED)
 }
 
 // Record the color from Tux Paint:
-void pixels_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b)
+void pixels_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g,
+                      Uint8 b, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
   pixels_r = r;
   pixels_g = g;
@@ -202,13 +215,13 @@ int pixels_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE
   return 1;
 }
 
-void pixels_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                     int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void pixels_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void pixels_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                      int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void pixels_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -216,3 +229,25 @@ int pixels_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_PAINT);
 }
+
+
+Uint8 pixels_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+Uint8 pixels_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 1;
+}
+
+void pixels_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                     SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  int canv_area_scale;
+
+  canv_area_scale = sqrt(canvas->w * canvas->h) / 144;
+
+  pixel_size = pow(2, size) * canv_area_scale;
+}
