@@ -6,9 +6,9 @@
 
   Author: Adam 'foo-script' Rakowski ; foo-script@o2.pl
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
+  Last updated: February 12, 2023
 */
 
 
@@ -34,7 +34,7 @@
 #include "SDL_image.h"
 #include "SDL_mixer.h"
 
-#define RATIO 5                 //change this value to get bigger puzzle
+static int RATIO = 5;
 
 //TODO: Fullscreen mode
 //In fullscreen mode RATIO _should_ be 1
@@ -50,7 +50,7 @@ static int rects_w, rects_h;
 SDL_Surface *canvas_backup;
 
 Uint32 puzzle_api_version(void);
-int puzzle_init(magic_api * api);
+int puzzle_init(magic_api * api, Uint32 disabled_features);
 int puzzle_get_tool_count(magic_api * api);
 SDL_Surface *puzzle_get_icon(magic_api * api, int which);
 char *puzzle_get_name(magic_api * api, int which);
@@ -59,7 +59,8 @@ char *puzzle_get_description(magic_api * api, int which, int mode);
 void puzzle_release(magic_api * api, int which,
                     SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void puzzle_shutdown(magic_api * api);
-void puzzle_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void puzzle_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                      SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int puzzle_requires_colors(magic_api * api, int which);
 void puzzle_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void puzzle_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
@@ -71,17 +72,21 @@ void puzzle_drag(magic_api * api, int which, SDL_Surface * canvas,
 void puzzle_click(magic_api * api, int which, int mode,
                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 int gcd(int a, int b);
+Uint8 puzzle_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 puzzle_default_size(magic_api * api, int which, int mode);
+void puzzle_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                     SDL_Rect * update_rect);
 
 Uint32 puzzle_api_version(void)
 {
   return (TP_MAGIC_API_VERSION);
 }
 
-int puzzle_init(magic_api * api)
+int puzzle_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/puzzle.wav", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/puzzle.wav", api->data_directory);
   puzzle_snd = Mix_LoadWAV(fname);
 
   return 1;
@@ -96,7 +101,7 @@ SDL_Surface *puzzle_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/puzzle.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/puzzle.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -119,8 +124,10 @@ char *puzzle_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBU
   return strdup(gettext_noop("Click to make a puzzle in fullscreen mode."));
 }
 
-void puzzle_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void puzzle_release(magic_api * api ATTRIBUTE_UNUSED,
+                    int which ATTRIBUTE_UNUSED,
+                    SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                    SDL_Surface * last ATTRIBUTE_UNUSED,
                     int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -131,8 +138,10 @@ void puzzle_shutdown(magic_api * api ATTRIBUTE_UNUSED)
     Mix_FreeChunk(puzzle_snd);
 }
 
-void puzzle_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
-                      Uint8 b ATTRIBUTE_UNUSED)
+void puzzle_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                      Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                      SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -148,19 +157,20 @@ int gcd(int a, int b)           //greatest common divisor
   return gcd(b, a % b);
 }
 
-void puzzle_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas)
+void puzzle_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas)
 {
   puzzle_gcd = RATIO * gcd(canvas->w, canvas->h);
   rects_w = (unsigned int)canvas->w / puzzle_gcd;
   rects_h = (unsigned int)canvas->h / puzzle_gcd;
   canvas_backup =
-    SDL_CreateRGBSurface(SDL_ANYFORMAT, canvas->w, canvas->h, canvas->format->BitsPerPixel, canvas->format->Rmask,
+    SDL_CreateRGBSurface(SDL_SWSURFACE, canvas->w, canvas->h,
+                         canvas->format->BitsPerPixel, canvas->format->Rmask,
                          canvas->format->Gmask, canvas->format->Bmask, canvas->format->Amask);
 }
 
-void puzzle_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void puzzle_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
   SDL_FreeSurface(canvas_backup);
   canvas_backup = NULL;
@@ -187,52 +197,52 @@ static void puzzle_draw(void *ptr, int which_tool ATTRIBUTE_UNUSED,
   y = (y / puzzle_gcd) * puzzle_gcd;
 
   if (!api->touched(x, y))
+  {
+    srand(rand());
+
+    r = rand() % 4;
+
+    rect_that.x = x;
+    rect_that.y = y;
+
+    switch (r)
     {
-      srand(rand());
+    case 0:                    //upper
+      if (y > puzzle_gcd)
+        rect_that.y = y - puzzle_gcd;
 
-      r = rand() % 4;
+      break;
+    case 1:                    //right
+      if (x < canvas->w - puzzle_gcd)
+        rect_that.x = x - puzzle_gcd;
 
-      rect_that.x = x;
-      rect_that.y = y;
+      break;
+    case 2:                    //lower
+      if (y < canvas->h - puzzle_gcd)
+        rect_that.y = y - puzzle_gcd;
 
-      switch (r)
-        {
-        case 0:                //upper
-          if (y > puzzle_gcd)
-            rect_that.y = y - puzzle_gcd;
-
-          break;
-        case 1:                //right
-          if (x < canvas->w - puzzle_gcd)
-            rect_that.x = x - puzzle_gcd;
-
-          break;
-        case 2:                //lower
-          if (y < canvas->h - puzzle_gcd)
-            rect_that.y = y - puzzle_gcd;
-
-          break;
-        case 3:                //left
-          if (x > puzzle_gcd)
-            rect_that.x = x - puzzle_gcd;
-          break;
-        }
-
-      rect_this.x = x;
-      rect_this.y = y;
-      rect_this.h = rect_this.w = puzzle_gcd;
-      rect_that.h = rect_that.w = puzzle_gcd;
-
-
-      SDL_BlitSurface(canvas, &rect_this, canvas, &rect_that);
-      SDL_BlitSurface(canvas_backup, &rect_that, canvas, &rect_this);
-      api->playsound(puzzle_snd, (x * 255) / canvas->w, 255);
+      break;
+    case 3:                    //left
+      if (x > puzzle_gcd)
+        rect_that.x = x - puzzle_gcd;
+      break;
     }
+
+    rect_this.x = x;
+    rect_this.y = y;
+    rect_this.h = rect_this.w = puzzle_gcd;
+    rect_that.h = rect_that.w = puzzle_gcd;
+
+
+    SDL_BlitSurface(canvas, &rect_this, canvas, &rect_that);
+    SDL_BlitSurface(canvas_backup, &rect_that, canvas, &rect_this);
+    api->playsound(puzzle_snd, (x * 255) / canvas->w, 255);
+  }
 }
 
 void puzzle_drag(magic_api * api, int which, SDL_Surface * canvas,
-                 SDL_Surface * last, int ox ATTRIBUTE_UNUSED, int oy ATTRIBUTE_UNUSED, int x, int y,
-                 SDL_Rect * update_rect)
+                 SDL_Surface * last, int ox ATTRIBUTE_UNUSED,
+                 int oy ATTRIBUTE_UNUSED, int x, int y, SDL_Rect * update_rect)
 {
   puzzle_draw(api, which, canvas, last, x - puzzle_gcd / 2, y - puzzle_gcd / 2);
 
@@ -252,3 +262,21 @@ void puzzle_click(magic_api * api, int which, int mode ATTRIBUTE_UNUSED,
 {
   puzzle_drag(api, which, canvas, last, x, y, x, y, update_rect);
 }
+
+
+Uint8 puzzle_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 10;
+}
+
+Uint8 puzzle_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+void puzzle_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED,
+                     Uint8 size, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  RATIO = size + 1;
+  puzzle_switchin(api, which, mode, canvas);
+}
