@@ -6,9 +6,9 @@
 
   Credits: Andrew Corcoran <akanewbie@gmail.com>
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: April 23, 2023
 */
 
 #include <stdio.h>
@@ -45,8 +44,8 @@
 
 void rain_click(magic_api *, int, int, SDL_Surface *, SDL_Surface *, int, int, SDL_Rect *);
 
-static const int rain_SIZE = 30;
-static const int rain_AMOUNT = 200;
+static int rain_SIZE = 30;
+static int rain_AMOUNT = 200;
 
 enum
 {
@@ -78,7 +77,7 @@ const char *rain_descs[rain_NUM_TOOLS][2] = {
 };
 
 Uint32 rain_api_version(void);
-int rain_init(magic_api * api);
+int rain_init(magic_api * api, Uint32 disabled_features);
 int rain_get_tool_count(magic_api * api);
 SDL_Surface *rain_get_icon(magic_api * api, int which);
 char *rain_get_name(magic_api * api, int which);
@@ -88,16 +87,22 @@ static void do_rain_drop(void *ptr, int which, SDL_Surface * canvas, SDL_Surface
 static void rain_linecb(void *ptr, int which, SDL_Surface * canvas, SDL_Surface * last, int x, int y);
 void rain_drag(magic_api * api, int which, SDL_Surface * canvas,
                SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
-void rain_click(magic_api * api, int which, int mode,
-                SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void rain_release(magic_api * api, int which,
-                  SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void rain_click(magic_api * api, int which, int mode, SDL_Surface * canvas,
+                SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void rain_release(magic_api * api, int which, SDL_Surface * canvas,
+                  SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void rain_shutdown(magic_api * api);
-void rain_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void rain_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                    SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int rain_requires_colors(magic_api * api, int which);
 void rain_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void rain_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int rain_modes(magic_api * api, int which);
+Uint8 rain_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 rain_default_size(magic_api * api, int which, int mode);
+void rain_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                   SDL_Rect * update_rect);
+
 
 Uint32 rain_api_version(void)
 {
@@ -107,14 +112,14 @@ Uint32 rain_api_version(void)
 //Checks if a a pixel is inside a raindrop shape centered on the origin
 static int rain_inRainShape(double x, double y, double r)
 {
-  if (sqrt(x * x + y * y) < (r * pow(cos(atan2(x, y)), 10.0)))
-    {
-      return 1;
-    }
+  if (sqrt(x * x + y * y) < (r * pow(cos(atan2(x, y)), (float)(rain_SIZE / 3.0))))
+  {
+    return 1;
+  }
   return 0;
 }
 
-int rain_init(magic_api * api)
+int rain_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
 
   int i;
@@ -122,10 +127,10 @@ int rain_init(magic_api * api)
 
   //Load sounds
   for (i = 0; i < rain_NUM_TOOLS; i++)
-    {
-      snprintf(fname, sizeof(fname), "%s/sounds/magic/%s", api->data_directory, rain_snd_filenames[i]);
-      rain_snd_effect[i] = Mix_LoadWAV(fname);
-    }
+  {
+    snprintf(fname, sizeof(fname), "%ssounds/magic/%s", api->data_directory, rain_snd_filenames[i]);
+    rain_snd_effect[i] = Mix_LoadWAV(fname);
+  }
 
   return (1);
 }
@@ -163,8 +168,8 @@ char *rain_get_description(magic_api * api ATTRIBUTE_UNUSED, int which, int mode
 }
 
 // Do the effect:
-static void do_rain_drop(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
-                         SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
+static void do_rain_drop(void *ptr, int which ATTRIBUTE_UNUSED,
+                         SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
 {
   magic_api *api = (magic_api *) ptr;
 
@@ -172,19 +177,19 @@ static void do_rain_drop(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * ca
   Uint8 r, g, b;
 
   for (yy = y - rain_SIZE / 2; yy < y + rain_SIZE / 2; yy++)
+  {
+    for (xx = x - rain_SIZE; xx < x + rain_SIZE; xx++)
     {
-      for (xx = x - rain_SIZE; xx < x + rain_SIZE; xx++)
-        {
-          if (rain_inRainShape(xx - x, yy - y + rain_SIZE / 2, rain_SIZE))
-            {
-              //api->rgbtohsv(rain_r, rain_g, rain_b, &h, &s, &v);
-              //api->hsvtorgb(h, s, rain_weights[(yy-y)*((rain_SIZE*2) -1)+(xx-x)], &r, &g, &b);
-              SDL_GetRGB(api->getpixel(canvas, xx, yy), canvas->format, &r, &g, &b);
-              api->putpixel(canvas, xx, yy, SDL_MapRGB(canvas->format, clamp(0, r - 50, 255),
-                                                       clamp(0, g - 50, 255), clamp(0, b + 200, 255)));
-            }
-        }
+      if (rain_inRainShape(xx - x, yy - y + rain_SIZE / 2, rain_SIZE))
+      {
+        //api->rgbtohsv(rain_r, rain_g, rain_b, &h, &s, &v);
+        //api->hsvtorgb(h, s, rain_weights[(yy-y)*((rain_SIZE*2) -1)+(xx-x)], &r, &g, &b);
+        SDL_GetRGB(api->getpixel(canvas, xx, yy), canvas->format, &r, &g, &b);
+        api->putpixel(canvas, xx, yy,
+                      SDL_MapRGB(canvas->format, clamp(0, r - 50, 255), clamp(0, g - 50, 255), clamp(0, b + 200, 255)));
+      }
     }
+  }
 
 }
 
@@ -194,10 +199,10 @@ static void rain_linecb(void *ptr, int which, SDL_Surface * canvas, SDL_Surface
   SDL_Rect rect;
 
   if (rand() % 10 == 0)
-    {
-      rain_click(api, which, MODE_PAINT, canvas, last,
-                 x + (rand() % rain_SIZE * 2) - rain_SIZE, y + (rand() % rain_SIZE * 2) - rain_SIZE, &rect);
-    }
+  {
+    rain_click(api, which, MODE_PAINT, canvas, last,
+               x + (rand() % rain_SIZE * 2) - rain_SIZE, y + (rand() % rain_SIZE * 2) - rain_SIZE, &rect);
+  }
 }
 
 // Affect the canvas on drag:
@@ -207,19 +212,19 @@ void rain_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->line((void *)api, which, canvas, last, ox, oy, x, y, 1, rain_linecb);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
   update_rect->x = ox - rain_SIZE * 2;
   update_rect->y = oy - rain_SIZE * 2;
@@ -233,39 +238,41 @@ void rain_click(magic_api * api, int which, int mode,
 {
 
   if (mode == MODE_PAINT)
-    {
-      do_rain_drop(api, which, canvas, last, x, y);
+  {
+    do_rain_drop(api, which, canvas, last, x, y);
 
-      update_rect->x = x - rain_SIZE;
-      update_rect->y = y - rain_SIZE;
-      update_rect->w = rain_SIZE * 2;
-      update_rect->h = rain_SIZE * 2;
+    update_rect->x = x - rain_SIZE;
+    update_rect->y = y - rain_SIZE;
+    update_rect->w = rain_SIZE * 2;
+    update_rect->h = rain_SIZE * 2;
 
-      api->playsound(rain_snd_effect[which], (x * 255) / canvas->w, 255);
-    }
+    api->playsound(rain_snd_effect[which], (x * 255) / canvas->w, 255);
+  }
   else
+  {
+
+    int i;
+
+    for (i = 0; i < rain_AMOUNT; i++)
     {
-
-      int i;
-
-      for (i = 0; i < rain_AMOUNT; i++)
-        {
-          do_rain_drop(api, which, canvas, last, rand() % canvas->w, rand() % canvas->h);
-        }
-
-      update_rect->x = 0;
-      update_rect->y = 0;
-      update_rect->w = canvas->w;
-      update_rect->h = canvas->h;
-
-      api->playsound(rain_snd_effect[which], 128, 255);
+      do_rain_drop(api, which, canvas, last, rand() % canvas->w, rand() % canvas->h);
     }
+
+    update_rect->x = 0;
+    update_rect->y = 0;
+    update_rect->w = canvas->w;
+    update_rect->h = canvas->h;
+
+    api->playsound(rain_snd_effect[which], 128, 255);
+  }
 }
 
 // Affect the canvas on release:
-void rain_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                  SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
-                  int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+void rain_release(magic_api * api ATTRIBUTE_UNUSED,
+                  int which ATTRIBUTE_UNUSED,
+                  SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                  SDL_Surface * last ATTRIBUTE_UNUSED, int x ATTRIBUTE_UNUSED,
+                  int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -276,17 +283,18 @@ void rain_shutdown(magic_api * api ATTRIBUTE_UNUSED)
   int i;
 
   for (i = 0; i < rain_NUM_TOOLS; i++)
+  {
+    if (rain_snd_effect[i] != NULL)
     {
-      if (rain_snd_effect[i] != NULL)
-        {
-          Mix_FreeChunk(rain_snd_effect[i]);
-        }
+      Mix_FreeChunk(rain_snd_effect[i]);
     }
+  }
 }
 
 // Record the color from Tux Paint:
-void rain_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
-                    Uint8 b ATTRIBUTE_UNUSED)
+void rain_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                    SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
+                    Uint8 b ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -297,13 +305,13 @@ int rain_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_U
 }
 
 
-void rain_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                   SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void rain_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                   int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void rain_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void rain_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                    int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -311,3 +319,23 @@ int rain_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_FULLSCREEN | MODE_PAINT);
 }
+
+
+Uint8 rain_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  /* Size affects both fullscreen and paint mode, in Rain tool! */
+  return 4;
+}
+
+Uint8 rain_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 2;
+}
+
+void rain_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                   SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                   SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  rain_SIZE = size * 15;
+  rain_AMOUNT = 400 / size;
+}
