@@ -6,9 +6,9 @@
 
   Credits: Adam 'foo-script' Rakowski <foo-script@o2.pl>
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
+  Last updated: April 19, 2023
 */
 
 // sound only plays on release
@@ -35,7 +35,8 @@
 #include "SDL_image.h"
 #include "SDL_mixer.h"
 #include <math.h>               //for sin, cos, ...
-#define ROSETTE_R 8             //circle's diameter
+
+static int ROSETTE_R = 8;       //circle's diameter
 
 static int xmid, ymid;
 
@@ -51,8 +52,9 @@ Mix_Chunk *rosette_snd;
 //                              Housekeeping functions
 
 Uint32 rosette_api_version(void);
-void rosette_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
-int rosette_init(magic_api * api);
+void rosette_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                       SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
+int rosette_init(magic_api * api, Uint32 disabled_features);
 int rosette_get_tool_count(magic_api * api);
 SDL_Surface *rosette_get_icon(magic_api * api, int which);
 char *rosette_get_name(magic_api * api, int which);
@@ -72,24 +74,32 @@ void rosette_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas
 void rosette_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int rosette_modes(magic_api * api, int which);
 void rosette_circle(void *ptr, int which, SDL_Surface * canvas, SDL_Surface * snapshot, int x, int y);
+Uint8 rosette_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 rosette_default_size(magic_api * api, int which, int mode);
+void rosette_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                      SDL_Rect * update_rect);
+
 
 Uint32 rosette_api_version(void)
 {
   return (TP_MAGIC_API_VERSION);
 }
 
-void rosette_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b)     //get the colors from API and store it in structure
+void rosette_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                       SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g,
+                       Uint8 b, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
+  //get the colors from API and store it in structure
   rosette_colors.r = r;
   rosette_colors.g = g;
   rosette_colors.b = b;
 }
 
-int rosette_init(magic_api * api)
+int rosette_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/picasso.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/picasso.ogg", api->data_directory);
   rosette_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -105,9 +115,9 @@ SDL_Surface *rosette_get_icon(magic_api * api, int which)
   char fname[1024];
 
   if (!which)
-    snprintf(fname, sizeof(fname), "%s/images/magic/rosette.png", api->data_directory);
+    snprintf(fname, sizeof(fname), "%simages/magic/rosette.png", api->data_directory);
   else
-    snprintf(fname, sizeof(fname), "%s/images/magic/picasso.png", api->data_directory);
+    snprintf(fname, sizeof(fname), "%simages/magic/picasso.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -138,8 +148,10 @@ int rosette_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUT
   return 1;
 }
 
-void rosette_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * snapshot ATTRIBUTE_UNUSED,
+void rosette_release(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                     SDL_Surface * snapshot ATTRIBUTE_UNUSED,
                      int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -177,37 +189,37 @@ void rosette_draw(void *ptr, int which, SDL_Surface * canvas, SDL_Surface * snap
   yy = (double)(y - ymid);
 
   if (which == 0)
-    {
-      angle = 2 * M_PI / 3;     //an angle between brushes
+  {
+    angle = 2 * M_PI / 3;       //an angle between brushes
 
-      x1 = (int)(xx * cos(angle) - yy * sin(angle));
-      y1 = (int)(xx * sin(angle) + yy * cos(angle));
+    x1 = (int)(xx * cos(angle) - yy * sin(angle));
+    y1 = (int)(xx * sin(angle) + yy * cos(angle));
 
-      x2 = (int)(xx * cos(2 * angle) - yy * sin(2 * angle));
-      y2 = (int)(xx * sin(2 * angle) + yy * cos(2 * angle));
-    }
+    x2 = (int)(xx * cos(2 * angle) - yy * sin(2 * angle));
+    y2 = (int)(xx * sin(2 * angle) + yy * cos(2 * angle));
+  }
   else
-    {
-      angle = atan(yy / xx);
+  {
+    angle = atan(yy / xx);
 
-      if ((xx < 0) && (yy > 0))
-        angle += M_PI;
+    if ((xx < 0) && (yy > 0))
+      angle += M_PI;
 
-      if ((xx < 0) && (yy < 0))
-        angle += M_PI;
+    if ((xx < 0) && (yy < 0))
+      angle += M_PI;
 
-      if ((xx > 0) && (yy < 0))
-        angle += 2 * M_PI;
+    if ((xx > 0) && (yy < 0))
+      angle += 2 * M_PI;
 
-      if ((y == ymid) && (xx < 0))
-        angle = M_PI;
+    if ((y == ymid) && (xx < 0))
+      angle = M_PI;
 
-      x1 = (int)(xx * cos(2 * angle) - yy * sin(2 * angle));
-      y1 = (int)(xx * sin(2 * angle) - yy * cos(angle));
+    x1 = (int)(xx * cos(2 * angle) - yy * sin(2 * angle));
+    y1 = (int)(xx * sin(2 * angle) - yy * cos(angle));
 
-      x2 = (int)(xx * cos(2 * angle) - yy * sin(2 * angle));
-      y2 = (int)(xx * sin(2 * angle) + yy * cos(2 * angle));
-    }
+    x2 = (int)(xx * cos(2 * angle) - yy * sin(2 * angle));
+    y2 = (int)(xx * sin(2 * angle) + yy * cos(2 * angle));
+  }
 
   rosette_circle(api, which, canvas, snapshot, x, y);
   rosette_circle(api, which, canvas, snapshot, (-1) * (x1 - xmid), y1 + ymid);
@@ -231,15 +243,15 @@ void rosette_click(magic_api * api, int which, int mode ATTRIBUTE_UNUSED,
   rosette_drag(api, which, canvas, last, x, y, x, y, update_rect);
 }
 
-void rosette_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void rosette_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
   xmid = canvas->w / 2;
   ymid = canvas->h / 2;
 }
 
-void rosette_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                       SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void rosette_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                       int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 
 }
@@ -248,3 +260,21 @@ int rosette_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_PAINT);
 }
+
+
+Uint8 rosette_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+Uint8 rosette_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 1;
+}
+
+void rosette_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                      SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  ROSETTE_R = (size + 2) * 2;
+}
