@@ -6,9 +6,9 @@
 
   Credits: Andrew Corcoran <akanewbie@gmail.com>
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: May 6, 2021
-  $Id$
+  Last updated: April 19, 2023
 */
 
 #include <stdio.h>
@@ -54,7 +53,7 @@ enum
 
 static const int THRESHOLD = 50;
 
-static const int sharpen_RADIUS = 16;
+static int sharpen_RADIUS = 16;
 
 static const double SHARPEN = 0.5;
 
@@ -88,7 +87,7 @@ const char *sharpen_descs[sharpen_NUM_TOOLS][2] = {
 };
 
 Uint32 sharpen_api_version(void);
-int sharpen_init(magic_api * api);
+int sharpen_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED);
 int sharpen_get_tool_count(magic_api * api);
 SDL_Surface *sharpen_get_icon(magic_api * api, int which);
 char *sharpen_get_name(magic_api * api, int which);
@@ -100,18 +99,25 @@ static void do_sharpen_full(void *ptr, SDL_Surface * canvas, SDL_Surface * last,
 static void do_sharpen_brush(void *ptr, int which, SDL_Surface * canvas, SDL_Surface * last, int x, int y);
 void sharpen_drag(magic_api * api, int which, SDL_Surface * canvas,
                   SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
-void sharpen_click(magic_api * api, int which, int mode,
-                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void sharpen_click(magic_api * api, int which, int mode, SDL_Surface * canvas,
+                   SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 
 void sharpen_release(magic_api * api, int which,
                      SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 
 void sharpen_shutdown(magic_api * api);
-void sharpen_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void sharpen_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                       SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int sharpen_requires_colors(magic_api * api, int which);
 void sharpen_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void sharpen_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int sharpen_modes(magic_api * api, int which);
+Uint8 sharpen_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 sharpen_default_size(magic_api * api, int which, int mode);
+void sharpen_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                      SDL_Rect * update_rect);
+
+
 
 Uint32 sharpen_api_version(void)
 {
@@ -120,17 +126,17 @@ Uint32 sharpen_api_version(void)
 
 
 // No setup required:
-int sharpen_init(magic_api * api)
+int sharpen_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
 
   int i;
   char fname[1024];
 
   for (i = 0; i < sharpen_NUM_TOOLS; i++)
-    {
-      snprintf(fname, sizeof(fname), "%s/sounds/magic/%s", api->data_directory, sharpen_snd_filenames[i]);
-      sharpen_snd_effect[i] = Mix_LoadWAV(fname);
-    }
+  {
+    snprintf(fname, sizeof(fname), "%ssounds/magic/%s", api->data_directory, sharpen_snd_filenames[i]);
+    sharpen_snd_effect[i] = Mix_LoadWAV(fname);
+  }
 
   return (1);
 }
@@ -198,61 +204,63 @@ static void do_sharpen_pixel(void *ptr, int which, SDL_Surface * canvas, SDL_Sur
   sobel_1 = 0;
   sobel_2 = 0;
   for (i = -1; i < 2; i++)
+  {
+    for (j = -1; j < 2; j++)
     {
-      for (j = -1; j < 2; j++)
-        {
-          //No need to check if inside canvas, getpixel does it for us.
-          SDL_GetRGB(api->getpixel(last, x + i, y + j), last->format, &r1, &g1, &b1);
-          grey = sharpen_grey(r1, g1, b1);
-          sobel_1 += grey * sobel_weights_1[i + 1][j + 1];
-          sobel_2 += grey * sobel_weights_2[i + 1][j + 1];
-        }
+      //No need to check if inside canvas, getpixel does it for us.
+      SDL_GetRGB(api->getpixel(last, x + i, y + j), last->format, &r1, &g1, &b1);
+      grey = sharpen_grey(r1, g1, b1);
+      sobel_1 += grey * sobel_weights_1[i + 1][j + 1];
+      sobel_2 += grey * sobel_weights_2[i + 1][j + 1];
     }
+  }
 
   temp = sqrt(sobel_1 * sobel_1 + sobel_2 * sobel_2);
   temp = (temp / 1443) * 255.0;
 
   // set image to white where edge value is below THRESHOLD
   if (which == TOOL_TRACE)
+  {
+    if (temp < THRESHOLD)
     {
-      if (temp < THRESHOLD)
-        {
-          api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, 255, 255, 255));
-        }
+      api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, 255, 255, 255));
     }
+  }
   //Simply display the edge values - provides a nice black and white silhouette image
   else if (which == TOOL_SILHOUETTE)
-    {
-      api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, temp, temp, temp));
-    }
+  {
+    api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, temp, temp, temp));
+  }
   //Add the edge values to the original image, creating a more distinct jump in contrast at edges
   else if (which == TOOL_SHARPEN)
-    {
-      SDL_GetRGB(api->getpixel(last, x, y), last->format, &r1, &g1, &b1);
-      api->putpixel(canvas, x, y, SDL_MapRGB(canvas->format, clamp(0.0, r1 + SHARPEN * temp, 255.0),
-                                             clamp(0.0, g1 + SHARPEN * temp, 255.0),
-                                             clamp(0.0, b1 + SHARPEN * temp, 255.0)));
-    }
+  {
+    SDL_GetRGB(api->getpixel(last, x, y), last->format, &r1, &g1, &b1);
+    api->putpixel(canvas, x, y,
+                  SDL_MapRGB(canvas->format,
+                             clamp(0.0, r1 + SHARPEN * temp, 255.0),
+                             clamp(0.0, g1 + SHARPEN * temp, 255.0), clamp(0.0, b1 + SHARPEN * temp, 255.0)));
+  }
 }
 
 // Do the effect for the full image
 static void do_sharpen_full(void *ptr, SDL_Surface * canvas, SDL_Surface * last, int which)
 {
-  magic_api * api = (magic_api *) ptr;
+  magic_api *api = (magic_api *) ptr;
 
   int x, y;
 
   for (y = 0; y < last->h; y++)
+  {
+    if (y % 10 == 0)
     {
-      if (y % 10 == 0) {
-        api->update_progress_bar();
-      }
-
-      for (x = 0; x < last->w; x++)
-        {
-          do_sharpen_pixel(api, which, canvas, last, x, y);
-        }
+      api->update_progress_bar();
     }
+
+    for (x = 0; x < last->w; x++)
+    {
+      do_sharpen_pixel(api, which, canvas, last, x, y);
+    }
+  }
 }
 
 //do the effect for the brush
@@ -262,15 +270,15 @@ static void do_sharpen_brush(void *ptr, int which, SDL_Surface * canvas, SDL_Sur
   magic_api *api = (magic_api *) ptr;
 
   for (yy = y - sharpen_RADIUS; yy < y + sharpen_RADIUS; yy++)
+  {
+    for (xx = x - sharpen_RADIUS; xx < x + sharpen_RADIUS; xx++)
     {
-      for (xx = x - sharpen_RADIUS; xx < x + sharpen_RADIUS; xx++)
-        {
-          if (api->in_circle(xx - x, yy - y, sharpen_RADIUS) && !api->touched(xx, yy))
-            {
-              do_sharpen_pixel(api, which, canvas, last, xx, yy);
-            }
-        }
+      if (api->in_circle(xx - x, yy - y, sharpen_RADIUS) && !api->touched(xx, yy))
+      {
+        do_sharpen_pixel(api, which, canvas, last, xx, yy);
+      }
     }
+  }
 }
 
 // Affect the canvas on drag:
@@ -283,19 +291,19 @@ void sharpen_drag(magic_api * api, int which, SDL_Surface * canvas,
   api->playsound(sharpen_snd_effect[which], (x * 255) / canvas->w, 255);
 
   if (ox > x)
-    {
-      int tmp = ox;
+  {
+    int tmp = ox;
 
-      ox = x;
-      x = tmp;
-    }
+    ox = x;
+    x = tmp;
+  }
   if (oy > y)
-    {
-      int tmp = oy;
+  {
+    int tmp = oy;
 
-      oy = y;
-      y = tmp;
-    }
+    oy = y;
+    y = tmp;
+  }
 
   update_rect->x = ox - sharpen_RADIUS;
   update_rect->y = oy - sharpen_RADIUS;
@@ -310,19 +318,21 @@ void sharpen_click(magic_api * api, int which, int mode,
   if (mode == MODE_PAINT)
     sharpen_drag(api, which, canvas, last, x, y, x, y, update_rect);
   else
-    {
-      update_rect->x = 0;
-      update_rect->y = 0;
-      update_rect->w = canvas->w;
-      update_rect->h = canvas->h;
-      do_sharpen_full(api, canvas, last, which);
-      api->playsound(sharpen_snd_effect[which], 128, 255);
-    }
+  {
+    update_rect->x = 0;
+    update_rect->y = 0;
+    update_rect->w = canvas->w;
+    update_rect->h = canvas->h;
+    do_sharpen_full(api, canvas, last, which);
+    api->playsound(sharpen_snd_effect[which], 128, 255);
+  }
 }
 
 // Affect the canvas on release:
-void sharpen_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void sharpen_release(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                     SDL_Surface * last ATTRIBUTE_UNUSED,
                      int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -334,17 +344,19 @@ void sharpen_shutdown(magic_api * api ATTRIBUTE_UNUSED)
   int i;
 
   for (i = 0; i < sharpen_NUM_TOOLS; i++)
+  {
+    if (sharpen_snd_effect[i] != NULL)
     {
-      if (sharpen_snd_effect[i] != NULL)
-        {
-          Mix_FreeChunk(sharpen_snd_effect[i]);
-        }
+      Mix_FreeChunk(sharpen_snd_effect[i]);
     }
+  }
 }
 
 // Record the color from Tux Paint:
-void sharpen_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED,
-                       Uint8 b ATTRIBUTE_UNUSED)
+void sharpen_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                       SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                       Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                       SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -354,13 +366,13 @@ int sharpen_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUT
   return 0;
 }
 
-void sharpen_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void sharpen_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void sharpen_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                       SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void sharpen_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                       int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -368,3 +380,25 @@ int sharpen_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_FULLSCREEN | MODE_PAINT);
 }
+
+
+
+Uint8 sharpen_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  if (mode == MODE_PAINT)
+    return 8;
+  else
+    return 0;
+}
+
+Uint8 sharpen_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 4;
+}
+
+void sharpen_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                      SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  sharpen_RADIUS = size * 4;
+}
