@@ -4,9 +4,9 @@
   Shift Magic Tool Plugin
   Tux Paint - A simple drawing program for children.
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,8 +23,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 20, 2021
-  $Id$
+  Last updated: February 12, 2023
 */
 
 #include <stdio.h>
@@ -46,7 +45,7 @@ static Mix_Chunk *shift_snd;
 static void shift_doit(magic_api * api, int which, SDL_Surface * canvas,
                        SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect, int crosshairs);
 Uint32 shift_api_version(void);
-int shift_init(magic_api * api);
+int shift_init(magic_api * api, Uint32 disabled_features);
 int shift_get_tool_count(magic_api * api);
 SDL_Surface *shift_get_icon(magic_api * api, int which);
 char *shift_get_name(magic_api * api, int which);
@@ -54,18 +53,24 @@ int shift_get_group(magic_api * api, int which);
 char *shift_get_description(magic_api * api, int which, int mode);
 void shift_drag(magic_api * api, int which, SDL_Surface * canvas,
                 SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
-void shift_click(magic_api * api, int which, int mode,
-                 SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void shift_release(magic_api * api, int which,
-                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void shift_click(magic_api * api, int which, int mode, SDL_Surface * canvas,
+                 SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void shift_release(magic_api * api, int which, SDL_Surface * canvas,
+                   SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void shift_shutdown(magic_api * api);
-void shift_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void shift_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                     SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int shift_requires_colors(magic_api * api, int which);
 
 void shift_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void shift_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int shift_modes(magic_api * api, int which);
 
+Uint8 shift_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 shift_default_size(magic_api * api, int which, int mode);
+void shift_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                    SDL_Rect * update_rect);
+
 
 
 Uint32 shift_api_version(void)
@@ -75,11 +80,11 @@ Uint32 shift_api_version(void)
 
 
 // No setup required:
-int shift_init(magic_api * api)
+int shift_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/shift.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/shift.ogg", api->data_directory);
   shift_snd = Mix_LoadWAV(fname);
 
   return (1);
@@ -96,7 +101,7 @@ SDL_Surface *shift_get_icon(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUT
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/shift.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/shift.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -130,9 +135,10 @@ void shift_drag(magic_api * api, int which, SDL_Surface * canvas,
   shift_doit(api, which, canvas, last, ox, oy, x, y, update_rect, 1);
 }
 
-static void shift_doit(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
-                       SDL_Surface * last, int ox ATTRIBUTE_UNUSED, int oy ATTRIBUTE_UNUSED, int x, int y,
-                       SDL_Rect * update_rect, int crosshairs)
+static void shift_doit(magic_api * api ATTRIBUTE_UNUSED,
+                       int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
+                       SDL_Surface * last, int ox ATTRIBUTE_UNUSED,
+                       int oy ATTRIBUTE_UNUSED, int x, int y, SDL_Rect * update_rect, int crosshairs)
 {
   SDL_Rect dest;
   int dx, dy;
@@ -162,123 +168,123 @@ static void shift_doit(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNU
 
 
   if (dy > 0)
+  {
+    if (dx > 0)
     {
-      if (dx > 0)
-        {
-          /* Top Left */
+      /* Top Left */
 
-          dest.x = dx - canvas->w;
-          dest.y = dy - canvas->h;
-
-          SDL_BlitSurface(last, NULL, canvas, &dest);
-        }
-
-
-      /* Top */
-
-      dest.x = dx;
+      dest.x = dx - canvas->w;
       dest.y = dy - canvas->h;
 
       SDL_BlitSurface(last, NULL, canvas, &dest);
-
-
-      if (dx < 0)
-        {
-          /* Top Right */
-
-          dest.x = dx + canvas->w;
-          dest.y = dy - canvas->h;
-
-          SDL_BlitSurface(last, NULL, canvas, &dest);
-        }
     }
 
 
+    /* Top */
+
+    dest.x = dx;
+    dest.y = dy - canvas->h;
+
+    SDL_BlitSurface(last, NULL, canvas, &dest);
+
+
+    if (dx < 0)
+    {
+      /* Top Right */
+
+      dest.x = dx + canvas->w;
+      dest.y = dy - canvas->h;
+
+      SDL_BlitSurface(last, NULL, canvas, &dest);
+    }
+  }
+
+
   if (dx > 0)
-    {
-      /* Left */
+  {
+    /* Left */
 
-      dest.x = dx - canvas->w;
-      dest.y = dy;
+    dest.x = dx - canvas->w;
+    dest.y = dy;
 
-      SDL_BlitSurface(last, NULL, canvas, &dest);
-    }
+    SDL_BlitSurface(last, NULL, canvas, &dest);
+  }
 
   if (dx < 0)
-    {
-      /* Right */
+  {
+    /* Right */
 
-      dest.x = dx + canvas->w;
-      dest.y = dy;
+    dest.x = dx + canvas->w;
+    dest.y = dy;
 
-      SDL_BlitSurface(last, NULL, canvas, &dest);
-    }
+    SDL_BlitSurface(last, NULL, canvas, &dest);
+  }
 
 
   if (dy < 0)
+  {
+    if (dx > 0)
     {
-      if (dx > 0)
-        {
-          /* Bottom Left */
+      /* Bottom Left */
 
-          dest.x = dx - canvas->w;
-          dest.y = dy + canvas->h;
-
-          SDL_BlitSurface(last, NULL, canvas, &dest);
-        }
-
-
-      /* Bottom */
-
-      dest.x = dx;
+      dest.x = dx - canvas->w;
       dest.y = dy + canvas->h;
 
       SDL_BlitSurface(last, NULL, canvas, &dest);
-
-
-      if (dx < 0)
-        {
-          /* Bottom Right */
-
-          dest.x = dx + canvas->w;
-          dest.y = dy + canvas->h;
-
-          SDL_BlitSurface(last, NULL, canvas, &dest);
-        }
     }
 
 
+    /* Bottom */
+
+    dest.x = dx;
+    dest.y = dy + canvas->h;
+
+    SDL_BlitSurface(last, NULL, canvas, &dest);
+
+
+    if (dx < 0)
+    {
+      /* Bottom Right */
+
+      dest.x = dx + canvas->w;
+      dest.y = dy + canvas->h;
+
+      SDL_BlitSurface(last, NULL, canvas, &dest);
+    }
+  }
+
+
   if (crosshairs)
-    {
-      dest.x = (canvas->w / 2) - 1;
-      dest.y = 0;
-      dest.w = 3;
-      dest.h = canvas->h;
+  {
+    dest.x = (canvas->w / 2) - 1;
+    dest.y = 0;
+    dest.w = 3;
+    dest.h = canvas->h;
 
-      SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 255, 255, 255));
+    SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 255, 255, 255));
 
-      dest.x = 0;
-      dest.y = (canvas->h / 2) - 1;
-      dest.w = canvas->w;
-      dest.h = 3;
+    dest.x = 0;
+    dest.y = (canvas->h / 2) - 1;
+    dest.w = canvas->w;
+    dest.h = 3;
 
-      SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 255, 255, 255));
+    SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 255, 255, 255));
 
 
-      dest.x = canvas->w / 2;
-      dest.y = 0;
-      dest.w = 1;
-      dest.h = canvas->h;
+    dest.x = canvas->w / 2;
+    dest.y = 0;
+    dest.w = 1;
+    dest.h = canvas->h;
 
-      SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 0, 0, 0));
+    SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 0, 0, 0));
 
-      dest.x = 0;
-      dest.y = canvas->h / 2;
-      dest.w = canvas->w;
-      dest.h = 1;
+    dest.x = 0;
+    dest.y = canvas->h / 2;
+    dest.w = canvas->w;
+    dest.h = 1;
 
-      SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 0, 0, 0));
-    }
+    SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 0, 0, 0));
+  }
 
 
   /* Update everything! */
@@ -318,8 +324,10 @@ void shift_shutdown(magic_api * api ATTRIBUTE_UNUSED)
 }
 
 // Record the color from Tux Paint:
-void shift_set_color(magic_api * api ATTRIBUTE_UNUSED,
-                     Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED)
+void shift_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                     SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                     Uint8 r ATTRIBUTE_UNUSED, Uint8 g ATTRIBUTE_UNUSED, Uint8 b ATTRIBUTE_UNUSED,
+                     SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
 
@@ -329,13 +337,13 @@ int shift_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_
   return 0;
 }
 
-void shift_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                    SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void shift_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                    int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void shift_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                     SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void shift_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                     int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -343,3 +351,20 @@ int shift_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_PAINT_WITH_PREVIEW);
 }
+
+
+Uint8 shift_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+Uint8 shift_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+void shift_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                    SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                    Uint8 size ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+}
