@@ -6,9 +6,9 @@
 
   Credits: Andrew Corcoran <akanewbie@gmail.com>
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: February 12, 2023
 */
 
 #include <stdio.h>
@@ -45,7 +44,7 @@
 
 double pi;
 static Uint8 toothpaste_r, toothpaste_g, toothpaste_b;
-static const int toothpaste_RADIUS = 10;
+static int toothpaste_RADIUS = 10;
 double *toothpaste_weights = NULL;
 
 enum
@@ -78,7 +77,7 @@ const char *toothpaste_descs[toothpaste_NUM_TOOLS] = {
 
 
 Uint32 toothpaste_api_version(void);
-int toothpaste_init(magic_api * api);
+int toothpaste_init(magic_api * api, Uint32 disabled_features);
 int toothpaste_get_tool_count(magic_api * api);
 SDL_Surface *toothpaste_get_icon(magic_api * api, int which);
 char *toothpaste_get_name(magic_api * api, int which);
@@ -89,14 +88,21 @@ void toothpaste_drag(magic_api * api, int which, SDL_Surface * canvas,
                      SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
 void toothpaste_click(magic_api * api, int which, int mode,
                       SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void toothpaste_release(magic_api * api, int which,
-                        SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void toothpaste_release(magic_api * api, int which, SDL_Surface * canvas,
+                        SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 void toothpaste_shutdown(magic_api * api);
-void toothpaste_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void toothpaste_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                          SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int toothpaste_requires_colors(magic_api * api, int which);
 void toothpaste_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void toothpaste_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int toothpaste_modes(magic_api * api, int which);
+int toothpaste_setup_weights(magic_api * api);
+Uint8 toothpaste_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 toothpaste_default_size(magic_api * api, int which, int mode);
+void toothpaste_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                         SDL_Rect * update_rect);
+
 
 Uint32 toothpaste_api_version(void)
 {
@@ -104,39 +110,52 @@ Uint32 toothpaste_api_version(void)
 }
 
 
-int toothpaste_init(magic_api * api)
+int toothpaste_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
-
   int i;
   char fname[1024];
-  int k, j;
 
   //Load sounds
   for (i = 0; i < toothpaste_NUM_TOOLS; i++)
-    {
-      snprintf(fname, sizeof(fname), "%s/sounds/magic/%s", api->data_directory, toothpaste_snd_filenames[i]);
-      toothpaste_snd_effect[i] = Mix_LoadWAV(fname);
-    }
+  {
+    snprintf(fname, sizeof(fname), "%ssounds/magic/%s", api->data_directory, toothpaste_snd_filenames[i]);
+    toothpaste_snd_effect[i] = Mix_LoadWAV(fname);
+  }
+
+  return (toothpaste_setup_weights(api));
+}
+
+int toothpaste_setup_weights(magic_api * api)
+{
+  int k, j;
 
   //Set up weights
   pi = acos(0.0) * 2;
+  if (toothpaste_weights != NULL)
+  {
+    free(toothpaste_weights);
+  }
+
   toothpaste_weights = (double *)malloc(toothpaste_RADIUS * 2 * toothpaste_RADIUS * 2 * sizeof(double));
   if (toothpaste_weights == NULL)
-    {
-      return (0);
-    }
+  {
+    return (0);
+  }
 
   for (k = -toothpaste_RADIUS; k < +toothpaste_RADIUS; k++)
+  {
+    for (j = -toothpaste_RADIUS; j < +toothpaste_RADIUS; j++)
     {
-      for (j = -toothpaste_RADIUS; j < +toothpaste_RADIUS; j++)
-        {
-          if (api->in_circle(j, k, toothpaste_RADIUS))
-            {
-              toothpaste_weights[(k + toothpaste_RADIUS) * ((toothpaste_RADIUS * 2) - 1) + (j + toothpaste_RADIUS)] =
-                ((fabs(atan2((double)(j), (double)(k)))) / pi);
-            }
-        }
+      if (api->in_circle(j, k, toothpaste_RADIUS))
+      {
+        toothpaste_weights[(k +
+                            toothpaste_RADIUS) * ((toothpaste_RADIUS * 2) -
+                                                  1) + (j +
+                                                        toothpaste_RADIUS)] =
+          ((fabs(atan2((double)(j), (double)(k)))) / pi);
+      }
     }
+  }
 
   return (1);
 }
@@ -174,33 +193,36 @@ char *toothpaste_get_description(magic_api * api ATTRIBUTE_UNUSED, int which, in
 }
 
 // Do the effect:
-static void do_toothpaste(void *ptr, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
-                          SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
+static void do_toothpaste(void *ptr, int which ATTRIBUTE_UNUSED,
+                          SDL_Surface * canvas, SDL_Surface * last ATTRIBUTE_UNUSED, int x, int y)
 {
   magic_api *api = (magic_api *) ptr;
-
   int xx, yy;
 
+  if (toothpaste_weights == NULL)
+    return;
+
   // double colr;
   float h, s, v;
   Uint8 r, g, b;
 
   for (yy = y - toothpaste_RADIUS; yy < y + toothpaste_RADIUS; yy++)
+  {
+    for (xx = x - toothpaste_RADIUS; xx < x + toothpaste_RADIUS; xx++)
     {
-      for (xx = x - toothpaste_RADIUS; xx < x + toothpaste_RADIUS; xx++)
-        {
-          if (api->in_circle(xx - x, yy - y, toothpaste_RADIUS) && !api->touched(xx, yy))
-            {
+      if (api->in_circle(xx - x, yy - y, toothpaste_RADIUS) && !api->touched(xx, yy))
+      {
 
-              api->rgbtohsv(toothpaste_r, toothpaste_g, toothpaste_b, &h, &s, &v);
-              api->hsvtorgb(h, s,
-                            toothpaste_weights[(yy - y + toothpaste_RADIUS) * ((toothpaste_RADIUS * 2) - 1) +
-                                               (xx - x + toothpaste_RADIUS)], &r, &g, &b);
-              api->putpixel(canvas, xx, yy, SDL_MapRGB(canvas->format, r, g, b));
+        api->rgbtohsv(toothpaste_r, toothpaste_g, toothpaste_b, &h, &s, &v);
+        api->hsvtorgb(h, s,
+                      toothpaste_weights[(yy - y +
+                                          toothpaste_RADIUS) *
+                                         ((toothpaste_RADIUS * 2) - 1) + (xx - x + toothpaste_RADIUS)], &r, &g, &b);
+        api->putpixel(canvas, xx, yy, SDL_MapRGB(canvas->format, r, g, b));
 
-            }
-        }
+      }
     }
+  }
 
 }
 
@@ -229,8 +251,10 @@ void toothpaste_click(magic_api * api, int which, int mode ATTRIBUTE_UNUSED,
 }
 
 // Affect the canvas on release:
-void toothpaste_release(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
-                        SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+void toothpaste_release(magic_api * api ATTRIBUTE_UNUSED,
+                        int which ATTRIBUTE_UNUSED,
+                        SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                        SDL_Surface * last ATTRIBUTE_UNUSED,
                         int x ATTRIBUTE_UNUSED, int y ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
 }
@@ -242,21 +266,23 @@ void toothpaste_shutdown(magic_api * api ATTRIBUTE_UNUSED)
   int i;
 
   for (i = 0; i < toothpaste_NUM_TOOLS; i++)
+  {
+    if (toothpaste_snd_effect[i] != NULL)
     {
-      if (toothpaste_snd_effect[i] != NULL)
-        {
-          Mix_FreeChunk(toothpaste_snd_effect[i]);
-        }
+      Mix_FreeChunk(toothpaste_snd_effect[i]);
     }
+  }
   if (toothpaste_weights != NULL)
-    {
-      free(toothpaste_weights);
-      toothpaste_weights = NULL;
-    }
+  {
+    free(toothpaste_weights);
+    toothpaste_weights = NULL;
+  }
 }
 
 // Record the color from Tux Paint:
-void toothpaste_set_color(magic_api * api ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b)
+void toothpaste_set_color(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED,
+                          SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g,
+                          Uint8 b, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
   toothpaste_r = r;
   toothpaste_g = g;
@@ -270,13 +296,13 @@ int toothpaste_requires_colors(magic_api * api ATTRIBUTE_UNUSED, int which ATTRI
 }
 
 
-void toothpaste_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                         SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void toothpaste_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                         int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void toothpaste_switchout(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                          SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void toothpaste_switchout(magic_api * api ATTRIBUTE_UNUSED,
+                          int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
@@ -284,3 +310,22 @@ int toothpaste_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSE
 {
   return (MODE_PAINT);
 }
+
+
+Uint8 toothpaste_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 6;
+}
+
+Uint8 toothpaste_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 2;
+}
+
+void toothpaste_set_size(magic_api * api, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                         SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 size,
+                         SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+  toothpaste_RADIUS = size * 5;
+  toothpaste_setup_weights(api);
+}
