@@ -4,9 +4,9 @@
   Tornado Magic Tool Plugin
   Tux Paint - A simple drawing program for children.
 
-  Copyright (c) 2002-2021 by Bill Kendrick and others; see AUTHORS.txt
+  Copyright (c) 2002-2023 by Bill Kendrick and others; see AUTHORS.txt
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   Some modifications to convert the flower plugin in to a tornado
   plugin by Pere Pujal i Carabantes
@@ -27,8 +27,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  Last updated: September 21, 2021
-  $Id$
+  Last updated: April 19, 2023
 */
 
 #include <stdio.h>
@@ -66,40 +65,39 @@ typedef struct
 
 static void tornado_predrag(magic_api * api, SDL_Surface * canvas, SDL_Surface * last, int ox, int oy, int x, int y);
 static void tornado_drawbase(magic_api * api, SDL_Surface * canvas);
-static void tornado_drawstalk(magic_api * api, SDL_Surface * canvas, SDL_Surface * last,
-                              int top_x, int top_y, int minx, int maxx, int bottom_x, int bottom_y, int final);
+static void tornado_drawstalk(magic_api * api, SDL_Surface * canvas,
+                              SDL_Surface * last, int top_x, int top_y,
+                              int minx, int maxx, int bottom_x, int bottom_y, int final);
 static void tornado_drawtornado(magic_api * api, SDL_Surface * canvas, int x, int y);
 static Point2D tornado_PointOnCubicBezier(Point2D * cp, float t);
 static void tornado_ComputeBezier(Point2D * cp, int numberOfPoints, Point2D * curve);
 static void tornado_colorize_cloud(magic_api * api);
 static Uint32 tornado_mess(Uint32 pixel, SDL_Surface * canvas);
 Uint32 tornado_api_version(void);
-int tornado_init(magic_api * api);
+int tornado_init(magic_api * api, Uint32 disabled_features);
 int tornado_get_tool_count(magic_api * api);
 SDL_Surface *tornado_get_icon(magic_api * api, int which);
 char *tornado_get_name(magic_api * api, int which);
 int tornado_get_group(magic_api * api, int which);
-
 char *tornado_get_description(magic_api * api, int which, int mode);
-
-
-
-
-
-
 void tornado_drag(magic_api * api, int which, SDL_Surface * canvas,
                   SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect);
-void tornado_click(magic_api * api, int which, int mode,
-                   SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
-void tornado_release(magic_api * api, int which,
-                     SDL_Surface * canvas, SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void tornado_click(magic_api * api, int which, int mode, SDL_Surface * canvas,
+                   SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
+void tornado_release(magic_api * api, int which, SDL_Surface * canvas,
+                     SDL_Surface * last, int x, int y, SDL_Rect * update_rect);
 
 void tornado_shutdown(magic_api * api);
-void tornado_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b);
+void tornado_set_color(magic_api * api, int which, SDL_Surface * canvas,
+                       SDL_Surface * last, Uint8 r, Uint8 g, Uint8 b, SDL_Rect * update_rect);
 int tornado_requires_colors(magic_api * api, int which);
 void tornado_switchin(magic_api * api, int which, int mode, SDL_Surface * canvas);
 void tornado_switchout(magic_api * api, int which, int mode, SDL_Surface * canvas);
 int tornado_modes(magic_api * api, int which);
+Uint8 tornado_accepted_sizes(magic_api * api, int which, int mode);
+Uint8 tornado_default_size(magic_api * api, int which, int mode);
+void tornado_set_size(magic_api * api, int which, int mode, SDL_Surface * canvas, SDL_Surface * last, Uint8 size,
+                      SDL_Rect * update_rect);
 
 
 
@@ -110,23 +108,23 @@ Uint32 tornado_api_version(void)
 
 
 // No setup required:
-int tornado_init(magic_api * api)
+int tornado_init(magic_api * api, Uint32 disabled_features ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
 /*
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/tornado_click.ogg",
+  snprintf(fname, sizeof(fname), "%ssounds/magic/tornado_click.ogg",
 	    api->data_directory);
   tornado_click_snd = Mix_LoadWAV(fname);
 */
 
-  snprintf(fname, sizeof(fname), "%s/sounds/magic/tornado_release.ogg", api->data_directory);
+  snprintf(fname, sizeof(fname), "%ssounds/magic/tornado_release.ogg", api->data_directory);
   tornado_release_snd = Mix_LoadWAV(fname);
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/tornado_base.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/tornado_base.png", api->data_directory);
   tornado_base = IMG_Load(fname);
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/tornado_cloud.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/tornado_cloud.png", api->data_directory);
   tornado_cloud = IMG_Load(fname);
 
   return (1);
@@ -143,7 +141,7 @@ SDL_Surface *tornado_get_icon(magic_api * api, int which ATTRIBUTE_UNUSED)
 {
   char fname[1024];
 
-  snprintf(fname, sizeof(fname), "%s/images/magic/tornado.png", api->data_directory);
+  snprintf(fname, sizeof(fname), "%simages/magic/tornado.png", api->data_directory);
 
   return (IMG_Load(fname));
 }
@@ -167,7 +165,8 @@ char *tornado_get_description(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIB
 }
 
 // Affect the canvas on drag:
-static void tornado_predrag(magic_api * api ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED,
+static void tornado_predrag(magic_api * api ATTRIBUTE_UNUSED,
+                            SDL_Surface * canvas ATTRIBUTE_UNUSED,
                             SDL_Surface * last ATTRIBUTE_UNUSED, int ox, int oy, int x, int y)
 {
   if (x < tornado_min_x)
@@ -187,22 +186,22 @@ static void tornado_predrag(magic_api * api ATTRIBUTE_UNUSED, SDL_Surface * canv
   // Determine which way to bend first:
   //
   if (tornado_side_decided == 0)
+  {
+    if (x < tornado_bottom_x - 10)
     {
-      if (x < tornado_bottom_x - 10)
-        {
-          tornado_side_first = SIDE_LEFT;
-          tornado_side_decided = 1;
-        }
-      else if (x > tornado_bottom_x + 10)
-        {
-          tornado_side_first = SIDE_RIGHT;
-          tornado_side_decided = 1;
-        }
+      tornado_side_first = SIDE_LEFT;
+      tornado_side_decided = 1;
     }
+    else if (x > tornado_bottom_x + 10)
+    {
+      tornado_side_first = SIDE_RIGHT;
+      tornado_side_decided = 1;
+    }
+  }
 }
 
-void tornado_drag(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas,
-                  SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect)
+void tornado_drag(magic_api * api, int which ATTRIBUTE_UNUSED,
+                  SDL_Surface * canvas, SDL_Surface * last, int ox, int oy, int x, int y, SDL_Rect * update_rect)
 {
   tornado_predrag(api, canvas, last, ox, oy, x, y);
 
@@ -316,8 +315,9 @@ static Uint32 tornado_mess(Uint32 pixel, SDL_Surface * canvas)
                       (tornado_g + g + (Uint8) f * 2) / 4, (tornado_b + b + (Uint8) f * 2) / 4, a));
 }
 
-static void tornado_drawstalk(magic_api * api, SDL_Surface * canvas, SDL_Surface * last,
-                              int top_x, int top_y, int minx, int maxx, int bottom_x, int bottom_y, int final)
+static void tornado_drawstalk(magic_api * api, SDL_Surface * canvas,
+                              SDL_Surface * last, int top_x, int top_y,
+                              int minx, int maxx, int bottom_x, int bottom_y, int final)
 {
   Point2D control_points[4];
   Point2D *curve;
@@ -335,15 +335,15 @@ static void tornado_drawstalk(magic_api * api, SDL_Surface * canvas, SDL_Surface
   control_points[0].y = top_y;
 
   if (tornado_side_first == SIDE_LEFT)
-    {
-      control_points[1].x = minx;
-      control_points[2].x = maxx;
-    }
+  {
+    control_points[1].x = minx;
+    control_points[2].x = maxx;
+  }
   else
-    {
-      control_points[1].x = maxx;
-      control_points[2].x = minx;
-    }
+  {
+    control_points[1].x = maxx;
+    control_points[2].x = minx;
+  }
 
   control_points[1].y = ((bottom_y - top_y) / 3) + top_y;
   control_points[2].y = (((bottom_y - top_y) / 3) * 2) + top_y;
@@ -367,58 +367,57 @@ static void tornado_drawstalk(magic_api * api, SDL_Surface * canvas, SDL_Surface
   /* Draw the curve: */
 
   for (i = 0; i < n_points - 1; i++)
+  {
+    if (final == 0)
     {
-      if (final == 0)
-        {
-          dest.x = curve[i].x;
-          dest.y = curve[i].y;
-          dest.w = 2;
-          dest.h = 2;
-          SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 0, 0, 0));
-        }
-      else
-        {
-          ii = n_points - i;
-          /* min 10 pixels then ii^2 / 2000 or 4 * ii^2 / canvas->w,
-             don't let the top of funnel be wider than the half of canvas */
-          if (n_points * n_points / 2000 > canvas->w / 4)
-            ww = 4 * n_points * n_points / canvas->w;
-          else
-            ww = 2000;
-
-          left = min(curve[i].x, curve[i + 1].x) - 5 - ii * ii / ww;
-          right = max(curve[i].x, curve[i + 1].x) + 5 + ii * ii / ww;
-
-          dest.x = left;
-          dest.y = curve[i].y;
-          dest.w = right - left + 1;
-          dest.h = 2;
-        }
-
-      rotation += 3;
-      /* The body of the tornado: 3x 1y rotation + some random particles */
-      for (p = dest.x; p < dest.x + dest.w; p++)
-        {
-          if ((float)rand() * 100 / RAND_MAX > 10)
-            {
-              api->putpixel(canvas, p, dest.y, api->getpixel(last, dest.x + (p - dest.x + rotation) % dest.w, dest.y));
-            }
-          else
-            {
-              api->putpixel(canvas, p, dest.y,
-                            tornado_mess(api->getpixel(last, dest.x + (p - dest.x + rotation) % dest.w, dest.y),
-                                         canvas));
-            }
-        }
-
-      /* Some random particles flying around the tornado */
-      for (p = dest.x - dest.w * 20 / 100; p < dest.x + dest.w + dest.w * 20 / 100; p++)
-        {
-          if ((float)rand() * 100 / RAND_MAX < 5 && ((p < dest.x) || (p > dest.w)))
-            api->putpixel(canvas, p, dest.y,
-                          tornado_mess(api->getpixel(last, dest.x + (p - dest.x + rotation) % dest.w, dest.y), canvas));
-        }
+      dest.x = curve[i].x;
+      dest.y = curve[i].y;
+      dest.w = 2;
+      dest.h = 2;
+      SDL_FillRect(canvas, &dest, SDL_MapRGB(canvas->format, 0, 0, 0));
     }
+    else
+    {
+      ii = n_points - i;
+      /* min 10 pixels then ii^2 / 2000 or 4 * ii^2 / canvas->w,
+         don't let the top of funnel be wider than the half of canvas */
+      if (n_points * n_points / 2000 > canvas->w / 4)
+        ww = 4 * n_points * n_points / canvas->w;
+      else
+        ww = 2000;
+
+      left = min(curve[i].x, curve[i + 1].x) - 5 - ii * ii / ww;
+      right = max(curve[i].x, curve[i + 1].x) + 5 + ii * ii / ww;
+
+      dest.x = left;
+      dest.y = curve[i].y;
+      dest.w = right - left + 1;
+      dest.h = 2;
+    }
+
+    rotation += 3;
+    /* The body of the tornado: 3x 1y rotation + some random particles */
+    for (p = dest.x; p < dest.x + dest.w; p++)
+    {
+      if ((float)rand() * 100 / RAND_MAX > 10)
+      {
+        api->putpixel(canvas, p, dest.y, api->getpixel(last, dest.x + (p - dest.x + rotation) % dest.w, dest.y));
+      }
+      else
+      {
+        api->putpixel(canvas, p, dest.y,
+                      tornado_mess(api->getpixel(last, dest.x + (p - dest.x + rotation) % dest.w, dest.y), canvas));
+      }
+    }
+
+    /* Some random particles flying around the tornado */
+    for (p = dest.x - dest.w * 20 / 100; p < dest.x + dest.w + dest.w * 20 / 100; p++)
+    {
+      if ((float)rand() * 100 / RAND_MAX < 5 && ((p < dest.x) || (p > dest.w)))
+        api->putpixel(canvas, p, dest.y,
+                      tornado_mess(api->getpixel(last, dest.x + (p - dest.x + rotation) % dest.w, dest.y), canvas));
+    }
+  }
 
   free(curve);
 }
@@ -442,7 +441,9 @@ void tornado_shutdown(magic_api * api ATTRIBUTE_UNUSED)
 }
 
 // Record the color from Tux Paint:
-void tornado_set_color(magic_api * api, Uint8 r, Uint8 g, Uint8 b)
+void tornado_set_color(magic_api * api, int which ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED,
+                       SDL_Surface * last ATTRIBUTE_UNUSED, Uint8 r, Uint8 g, Uint8 b,
+                       SDL_Rect * update_rect ATTRIBUTE_UNUSED)
 {
   tornado_r = r;
   tornado_g = g;
@@ -546,28 +547,28 @@ static void tornado_colorize_cloud(magic_api * api)
   SDL_LockSurface(tornado_cloud_colorized);
 
   for (y = 0; y < tornado_cloud->h; y++)
+  {
+    for (x = 0; x < tornado_cloud->w; x++)
     {
-      for (x = 0; x < tornado_cloud->w; x++)
-        {
-          SDL_GetRGBA(api->getpixel(tornado_cloud, x, y), tornado_cloud->format, &r, &g, &b, &a);
+      SDL_GetRGBA(api->getpixel(tornado_cloud, x, y), tornado_cloud->format, &r, &g, &b, &a);
 
-          api->putpixel(tornado_cloud_colorized, x, y,
-                        SDL_MapRGBA(tornado_cloud_colorized->format,
-                                    (tornado_r + r * 2) / 3, (tornado_g + g * 2) / 3, (tornado_b + b * 2) / 3, a));
-        }
+      api->putpixel(tornado_cloud_colorized, x, y,
+                    SDL_MapRGBA(tornado_cloud_colorized->format,
+                                (tornado_r + r * 2) / 3, (tornado_g + g * 2) / 3, (tornado_b + b * 2) / 3, a));
     }
+  }
 
   SDL_UnlockSurface(tornado_cloud_colorized);
   SDL_UnlockSurface(tornado_cloud);
 }
 
-void tornado_switchin(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                      SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void tornado_switchin(magic_api * api ATTRIBUTE_UNUSED,
+                      int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
 }
 
-void tornado_switchout(magic_api * api, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
-                       SDL_Surface * canvas ATTRIBUTE_UNUSED)
+void tornado_switchout(magic_api * api, int which ATTRIBUTE_UNUSED,
+                       int mode ATTRIBUTE_UNUSED, SDL_Surface * canvas ATTRIBUTE_UNUSED)
 {
   api->stopsound();
 }
@@ -576,3 +577,20 @@ int tornado_modes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED)
 {
   return (MODE_PAINT_WITH_PREVIEW);
 }
+
+
+Uint8 tornado_accepted_sizes(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+Uint8 tornado_default_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED)
+{
+  return 0;
+}
+
+void tornado_set_size(magic_api * api ATTRIBUTE_UNUSED, int which ATTRIBUTE_UNUSED, int mode ATTRIBUTE_UNUSED,
+                      SDL_Surface * canvas ATTRIBUTE_UNUSED, SDL_Surface * last ATTRIBUTE_UNUSED,
+                      Uint8 size ATTRIBUTE_UNUSED, SDL_Rect * update_rect ATTRIBUTE_UNUSED)
+{
+}
