@@ -1,8 +1,8 @@
 /*
   fonts.h
 
-  Copyright (c) 2009-2017
-  http://www.tuxpaint.org/
+  Copyright (c) 2009-2023
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  $Id$
+  Last updated: June 13, 2023
 */
 
 #ifndef FONTS_H
@@ -31,19 +31,32 @@
 #if defined(WIN32) || defined(__BEOS__)
 #undef FORKED_FONTS
 #endif
-#ifdef __APPLE__
+#ifdef __ANDROID__
 #undef FORKED_FONTS
 #endif
 
-
 #include "SDL.h"
 #include "SDL_ttf.h"
+#include "SDL2_Pango.h"
 
-#ifndef NO_SDLPANGO
-#include "SDL_Pango.h"
-#endif
+#include "i18n.h"
 
-#define PANGO_DEFAULT_FONT "BitStream Vera"
+/* UI font, which as of 0.9.31, can be overridden by "uifont"
+   setting (also, this will be used if "uifont=default" is specified,
+   e.g. to override a config. file option using a command-line option).
+
+   Also, starting in 0.9.31, we can specify different preferred fonts
+   based on locale.
+*/
+
+typedef struct default_locale_font_s {
+  int locale_id;
+  const char * font_name;
+  const char * font_name_fallback;
+} default_locale_font_t;
+
+extern default_locale_font_t default_local_fonts[];
+extern const char * PANGO_DEFAULT_FONT, * PANGO_DEFAULT_FONT_FALLBACK;
 
 #include "compiler.h"
 
@@ -51,24 +64,6 @@
 #if !defined(FORKED_FONTS) && !defined(WIN32)
 #include "SDL_thread.h"
 #include "SDL_mutex.h"
-#else
-/* This shouldn't really be here :-)
- * Move into 'fonts.c' and the code in 'tuxpaint.c' 
- * that uses this lot should be put into 'fonts.c' as well.
- */
-
-#ifdef NO_SDLPANGO
-/* Only kill SDL_thread stuff when we're not using Pango, because we need it to let fontconfig make its cache (takes a long time the first time) -bjk 2010.04.27 */
-#define SDL_CreateThread(fn,vp) (void*)(long)(fn(vp))
-#define SDL_WaitThread(tid,rcp) do{(void)tid;(void)rcp;}while(0)
-#define SDL_Thread int
-#define SDL_mutex int
-#define SDL_CreateMutex() 0     // creates in released state
-#define SDL_DestroyMutex(lock)
-#define SDL_mutexP(lock)        // take lock
-#define SDL_mutexV(lock)        // release lock
-#endif
-
 #endif
 
 extern SDL_Thread *font_thread;
@@ -81,29 +76,18 @@ extern int font_socket_fd;
 extern int no_system_fonts;
 extern int all_locale_fonts;
 
-/* FIXME: SDL_ttf is up to 2.0.8, so we can probably fully remove this;
-   -bjk 2007.06.05 */
-/*
-TTF_Font *BUGFIX_TTF_OpenFont206(const char *const file, int ptsize);
-#define TTF_OpenFont    BUGFIX_TTF_OpenFont206
-*/
-
 
 /* Stuff that wraps either SDL_Pango or SDL_TTF for font rendering: */
 
 enum
 {
-#ifndef NO_SDLPANGO
   FONT_TYPE_PANGO,
-#endif
   FONT_TYPE_TTF
 };
 
 typedef struct TuxPaint_Font_s
 {
-#ifndef NO_SDLPANGO
   SDLPango_Context *pango_context;
-#endif
   int typ;
   TTF_Font *ttf_font;
   int height;
@@ -115,8 +99,9 @@ int TuxPaint_Font_FontHeight(TuxPaint_Font * tpf);
 
 #ifdef FORKED_FONTS
 void reliable_write(int fd, const void *buf, size_t count);
-void run_font_scanner(SDL_Surface * screen, const char *restrict const locale);
-void receive_some_font_info(SDL_Surface * screen);
+void run_font_scanner(SDL_Surface * screen, SDL_Texture * texture,
+                      SDL_Renderer * renderer, const char *restrict const locale);
+void receive_some_font_info(SDL_Surface * screen, SDL_Texture * texture, SDL_Renderer * renderer);
 #endif
 
 //////////////////////////////////////////////////////////////////////
@@ -188,13 +173,10 @@ TuxPaint_Font *TuxPaint_Font_OpenFont(const char *pangodesc, const char *ttffile
 void TuxPaint_Font_CloseFont(TuxPaint_Font * tpf);
 const char *TuxPaint_Font_FontFaceFamilyName(TuxPaint_Font * tpf);
 const char *TuxPaint_Font_FontFaceStyleName(TuxPaint_Font * tpf);
-
-#ifdef NO_SDLPANGO
-TuxPaint_Font *load_locale_font(TuxPaint_Font * fallback, int size);
-#else
 void sdl_color_to_pango_color(SDL_Color sdl_color, SDLPango_Matrix * pango_color);
-#endif
 
-int load_user_fonts(SDL_Surface * screen, void *vp, const char *restrict const locale);
+int load_user_fonts(SDL_Surface * screen, SDL_Texture * texture,
+                    SDL_Renderer * renderer, void *vp, const char *restrict const locale);
+char * ask_pango_for_font(char * pangodesc);
 
 #endif
