@@ -4,9 +4,9 @@
   For Tux Paint
   Language-related functions
 
-  Copyright (c) 2002-2020 by Bill Kendrick and others
+  Copyright (c) 2002-2023 by Bill Kendrick and others
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,9 +23,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  $Id$
-  
-  June 14, 2002 - July 26, 2020
+  June 14, 2002 - April 30, 2023
 */
 
 
@@ -206,10 +204,7 @@ extern w_langs wished_langs[255];
 /* Function prototypes: */
 
 int get_current_language(void);
-int setup_i18n(const char *restrict lang, const char *restrict locale, int * ptr_num_wished_languages) MUST_CHECK;
-
-#ifdef NO_SDLPANGO
-     int smash_i18n(void) MUST_CHECK;
-#endif
+int setup_i18n(const char *restrict lang, const char *restrict locale, int *ptr_num_wished_languages) MUST_CHECK;
+     void mysetenv(const char *name, const char *value);
 
 #endif
