@@ -14,15 +14,20 @@ struct cfginfo
   const char *colorfile;
   const char *datadir;
   const char *disable_label;
+  const char *disable_brushspacing;
   const char *disable_magic_controls;
+  const char *disable_magic_sizes;
   const char *disable_shape_controls;
   const char *disable_print;
   const char *disable_quit;
   const char *disable_save;
   const char *disable_screensaver;
   const char *disable_stamp_controls;
+  const char *disable_template_export;
+  const char *disable_erase;
   const char *dont_do_xor;
   const char *dont_load_stamps;
+  const char *no_stamp_rotation;
   const char *exportdir;
   const char *fullscreen;
   const char *grab_input;
@@ -32,6 +37,7 @@ struct cfginfo
   const char *native_screensize;
   const char *new_colors_last;
   const char *no_button_distinction;
+  const char *tp_ui_font;
   const char *no_fancy_cursors;
   const char *no_system_fonts;
   const char *noshortcuts;
@@ -47,6 +53,7 @@ struct cfginfo
   const char *colors_rows;
   const char *print_delay;
   const char *printcommand;
+  const char *reversesort;
 //  const char *promptless_save;
   const char *_promptless_save_over;
   const char *_promptless_save_over_new;
@@ -77,6 +84,7 @@ struct cfginfo
   const char *joystick_button_selectshapestool;
   const char *joystick_button_selecttexttool;
   const char *joystick_button_selectlabeltool;
+  const char *joystick_button_selectfilltool;
   const char *joystick_button_selectmagictool;
   const char *joystick_button_undo;
   const char *joystick_button_redo;
@@ -92,5 +100,5 @@ struct cfginfo
 
 #define CFGINFO_MAXOFFSET  (sizeof(struct cfginfo))
 
-extern void parse_one_option(struct cfginfo *restrict tmpcfg, const char *str, const char *opt,
-                             const char *restrict src);
+extern void parse_one_option(struct cfginfo *restrict tmpcfg, const char *str,
+                             const char *opt, const char *restrict src);
