@@ -24,7 +24,7 @@
   (See COPYING.txt)
 
   June 14, 2002 - February 17, 2006
-  $Id: pixels.c,v 1.4 2009/11/22 23:17:35 albert Exp $
+  $Id$
 */
 
 #include "pixels.h"
@@ -37,14 +37,12 @@ static void putpixel8(SDL_Surface * surface, int x, int y, Uint32 pixel)
   Uint8 *p;
 
   /* Assuming the X/Y values are within the bounds of this surface... */
-  if (likely
-      (likely((unsigned) x < (unsigned) surface->w)
-       && likely((unsigned) y < (unsigned) surface->h)))
+  if (likely(likely((unsigned)x < (unsigned)surface->w) && likely((unsigned)y < (unsigned)surface->h)))
   {
     // Set a pointer to the exact location in memory of the pixel
-    p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start: beginning of RAM */
-		   (y * surface->pitch) +	/* Go down Y lines */
-		   x);		/* Go in X pixels */
+    p = (Uint8 *) (((Uint8 *) surface->pixels) +        /* Start: beginning of RAM */
+                   (y * surface->pitch) +       /* Go down Y lines */
+                   x);          /* Go in X pixels */
 
 
     /* Set the (correctly-sized) piece of data in the surface's RAM
@@ -60,14 +58,12 @@ static void putpixel16(SDL_Surface * surface, int x, int y, Uint32 pixel)
   Uint8 *p;
 
   /* Assuming the X/Y values are within the bounds of this surface... */
-  if (likely
-      (likely((unsigned) x < (unsigned) surface->w)
-       && likely((unsigned) y < (unsigned) surface->h)))
+  if (likely(likely((unsigned)x < (unsigned)surface->w) && likely((unsigned)y < (unsigned)surface->h)))
   {
     // Set a pointer to the exact location in memory of the pixel
-    p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start: beginning of RAM */
-		   (y * surface->pitch) +	/* Go down Y lines */
-		   (x * 2));	/* Go in X pixels */
+    p = (Uint8 *) (((Uint8 *) surface->pixels) +        /* Start: beginning of RAM */
+                   (y * surface->pitch) +       /* Go down Y lines */
+                   (x * 2));    /* Go in X pixels */
 
 
     /* Set the (correctly-sized) piece of data in the surface's RAM
@@ -83,14 +79,12 @@ static void putpixel24(SDL_Surface * surface, int x, int y, Uint32 pixel)
   Uint8 *p;
 
   /* Assuming the X/Y values are within the bounds of this surface... */
-  if (likely
-      (likely((unsigned) x < (unsigned) surface->w)
-       && likely((unsigned) y < (unsigned) surface->h)))
+  if (likely(likely((unsigned)x < (unsigned)surface->w) && likely((unsigned)y < (unsigned)surface->h)))
   {
     // Set a pointer to the exact location in memory of the pixel
-    p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start: beginning of RAM */
-		   (y * surface->pitch) +	/* Go down Y lines */
-		   (x * 3));	/* Go in X pixels */
+    p = (Uint8 *) (((Uint8 *) surface->pixels) +        /* Start: beginning of RAM */
+                   (y * surface->pitch) +       /* Go down Y lines */
+                   (x * 3));    /* Go in X pixels */
 
 
     /* Set the (correctly-sized) piece of data in the surface's RAM
@@ -118,20 +112,18 @@ static void putpixel32(SDL_Surface * surface, int x, int y, Uint32 pixel)
   Uint8 *p;
 
   /* Assuming the X/Y values are within the bounds of this surface... */
-  if (likely
-      (likely((unsigned) x < (unsigned) surface->w)
-       && likely((unsigned) y < (unsigned) surface->h)))
+  if (likely(likely((unsigned)x < (unsigned)surface->w) && likely((unsigned)y < (unsigned)surface->h)))
   {
     // Set a pointer to the exact location in memory of the pixel
-    p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start: beginning of RAM */
-		   (y * surface->pitch) +	/* Go down Y lines */
-		   (x * 4));	/* Go in X pixels */
+    p = (Uint8 *) (((Uint8 *) surface->pixels) +        /* Start: beginning of RAM */
+                   (y * surface->pitch) +       /* Go down Y lines */
+                   (x * 4));    /* Go in X pixels */
 
 
     /* Set the (correctly-sized) piece of data in the surface's RAM
      *          to the pixel value sent in: */
 
-    *(Uint32 *) p = pixel;	// 32-bit display
+    *(Uint32 *) p = pixel;      // 32-bit display
   }
 }
 
@@ -141,17 +133,17 @@ static Uint32 getpixel8(SDL_Surface * surface, int x, int y)
   Uint8 *p;
 
   /* get the X/Y values within the bounds of this surface */
-  if (unlikely((unsigned) x > (unsigned) surface->w - 1u))
+  if (unlikely((unsigned)x > (unsigned)surface->w - 1u))
     x = (x < 0) ? 0 : surface->w - 1;
-  if (unlikely((unsigned) y > (unsigned) surface->h - 1u))
+  if (unlikely((unsigned)y > (unsigned)surface->h - 1u))
     y = (y < 0) ? 0 : surface->h - 1;
 
   /* Set a pointer to the exact location in memory of the pixel
      in question: */
 
-  p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start at top of RAM */
-		 (y * surface->pitch) +	/* Go down Y lines */
-		 x);		/* Go in X pixels */
+  p = (Uint8 *) (((Uint8 *) surface->pixels) +  /* Start at top of RAM */
+                 (y * surface->pitch) + /* Go down Y lines */
+                 x);            /* Go in X pixels */
 
 
   /* Return the correctly-sized piece of data containing the
@@ -167,17 +159,17 @@ static Uint32 getpixel16(SDL_Surface * surface, int x, int y)
   Uint8 *p;
 
   /* get the X/Y values within the bounds of this surface */
-  if (unlikely((unsigned) x > (unsigned) surface->w - 1u))
+  if (unlikely((unsigned)x > (unsigned)surface->w - 1u))
     x = (x < 0) ? 0 : surface->w - 1;
-  if (unlikely((unsigned) y > (unsigned) surface->h - 1u))
+  if (unlikely((unsigned)y > (unsigned)surface->h - 1u))
     y = (y < 0) ? 0 : surface->h - 1;
 
   /* Set a pointer to the exact location in memory of the pixel
      in question: */
 
-  p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start at top of RAM */
-		 (y * surface->pitch) +	/* Go down Y lines */
-		 (x * 2));	/* Go in X pixels */
+  p = (Uint8 *) (((Uint8 *) surface->pixels) +  /* Start at top of RAM */
+                 (y * surface->pitch) + /* Go down Y lines */
+                 (x * 2));      /* Go in X pixels */
 
 
   /* Return the correctly-sized piece of data containing the
@@ -194,17 +186,17 @@ static Uint32 getpixel24(SDL_Surface * surface, int x, int y)
   Uint32 pixel;
 
   /* get the X/Y values within the bounds of this surface */
-  if (unlikely((unsigned) x > (unsigned) surface->w - 1u))
+  if (unlikely((unsigned)x > (unsigned)surface->w - 1u))
     x = (x < 0) ? 0 : surface->w - 1;
-  if (unlikely((unsigned) y > (unsigned) surface->h - 1u))
+  if (unlikely((unsigned)y > (unsigned)surface->h - 1u))
     y = (y < 0) ? 0 : surface->h - 1;
 
   /* Set a pointer to the exact location in memory of the pixel
      in question: */
 
-  p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start at top of RAM */
-		 (y * surface->pitch) +	/* Go down Y lines */
-		 (x * 3));	/* Go in X pixels */
+  p = (Uint8 *) (((Uint8 *) surface->pixels) +  /* Start at top of RAM */
+                 (y * surface->pitch) + /* Go down Y lines */
+                 (x * 3));      /* Go in X pixels */
 
 
   /* Return the correctly-sized piece of data containing the
@@ -227,31 +219,31 @@ static Uint32 getpixel32(SDL_Surface * surface, int x, int y)
   Uint8 *p;
 
   /* get the X/Y values within the bounds of this surface */
-  if (unlikely((unsigned) x > (unsigned) surface->w - 1u))
+  if (unlikely((unsigned)x > (unsigned)surface->w - 1u))
     x = (x < 0) ? 0 : surface->w - 1;
-  if (unlikely((unsigned) y > (unsigned) surface->h - 1u))
+  if (unlikely((unsigned)y > (unsigned)surface->h - 1u))
     y = (y < 0) ? 0 : surface->h - 1;
 
   /* Set a pointer to the exact location in memory of the pixel
      in question: */
 
-  p = (Uint8 *) (((Uint8 *) surface->pixels) +	/* Start at top of RAM */
-		 (y * surface->pitch) +	/* Go down Y lines */
-		 (x * 4));	/* Go in X pixels */
+  p = (Uint8 *) (((Uint8 *) surface->pixels) +  /* Start at top of RAM */
+                 (y * surface->pitch) + /* Go down Y lines */
+                 (x * 4));      /* Go in X pixels */
 
 
   /* Return the correctly-sized piece of data containing the
    * pixel's value (an 8-bit palette value, or a 16-, 24- or 32-bit
    * RGB value) */
 
-  return *(Uint32 *) p;		// 32-bit display
+  return *(Uint32 *) p;         // 32-bit display
 }
 
-void (*putpixels[]) (SDL_Surface *, int, int, Uint32) =
-{
-putpixel8, putpixel8, putpixel16, putpixel24, putpixel32};
+void (*putpixels[])(SDL_Surface *, int, int, Uint32) = {
+  putpixel8, putpixel8, putpixel16, putpixel24, putpixel32
+};
 
 
-Uint32(*getpixels[])(SDL_Surface *, int, int) =
-{
-getpixel8, getpixel8, getpixel16, getpixel24, getpixel32};
+Uint32(*getpixels[])(SDL_Surface *, int, int) = {
+  getpixel8, getpixel8, getpixel16, getpixel24, getpixel32
+};
