@@ -6,9 +6,9 @@
   (for non-Windows, non-BeOS platforms, e.g. Linux and macOS)
   (moved from tuxpaint.c in 0.9.17)
 
-  Copyright (c) 2009 by Bill Kendrick and others
+  Copyright (c) 2007-2022 by Bill Kendrick and others
   bill@newbreedsoftware.com
-  http://www.tuxpaint.org/
+  https://tuxpaint.org/
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -35,8 +35,7 @@
     documentation.  This software is provided "as is" without express or
     implied warranty.
 
-
-  June 24, 2007 - January 29, 2009
+  June 24, 2007 - December 11, 2022
   $Id$
 */
 
@@ -102,38 +101,36 @@ int do_ps_save(FILE * fi,
 
   /* Determine paper size: */
 
-  paperinit();                  // FIXME: Should we do this at startup? -bjk 2007.06.25
-
   if (pprsize == NULL)
+  {
+    /* User did not request a specific paper size (on command-line or
+       in config file), ask the system.  It will return either their
+       $PAPER env. var., the value from /etc/papersize, or NULL: */
+
+    pprsize = systempapername();
+
+    if (pprsize == NULL)
     {
-      /* User did not request a specific paper size (on command-line or
-         in config file), ask the system.  It will return either their
-         $PAPER env. var., the value from /etc/papersize, or NULL: */
+      /* No setting, env. var. or /etc/ file; use the default! */
 
-      pprsize = systempapername();
-
-      if (pprsize == NULL)
-        {
-          /* No setting, env. var. or /etc/ file; use the default! */
-
-          pprsize = defaultpapername();
+      pprsize = defaultpapername();
 
 #ifdef DEBUG
-          printf("Using default paper\n");
-#endif
-        }
-#ifdef DEBUG
-      else
-        {
-          printf("Using system paper\n");
-        }
+      printf("Using default paper\n");
 #endif
     }
+#ifdef DEBUG
+    else
+    {
+      printf("Using system paper\n");
+    }
+#endif
+  }
 #ifdef DEBUG
   else
-    {
-      printf("Using user paper\n");
-    }
+  {
+    printf("Using user paper\n");
+  }
 #endif
 
 #ifdef DEBUG
@@ -158,17 +155,17 @@ int do_ps_save(FILE * fi,
   /* Determine whether it's best to rotate the image: */
 
   if ((ppr_w >= ppr_h && img_w >= img_h) || (ppr_w <= ppr_h && img_w <= img_h))
-    {
-      rotate = 0;
-      r_img_w = img_w;
-      r_img_h = img_h;
-    }
+  {
+    rotate = 0;
+    r_img_w = img_w;
+    r_img_h = img_h;
+  }
   else
-    {
-      rotate = 1;
-      r_img_w = img_h;
-      r_img_h = img_w;
-    }
+  {
+    rotate = 1;
+    r_img_w = img_h;
+    r_img_h = img_w;
+  }
 
 #ifdef DEBUG
   printf("Image is %d x %d\n", img_w, img_h);
@@ -263,23 +260,23 @@ int do_ps_save(FILE * fi,
   cur_line_len = 0;
 
   for (y = 0; y < img_h; y++)
+  {
+    for (plane = 0; plane < 3; plane++)
     {
-      for (plane = 0; plane < 3; plane++)
-        {
-          for (x = 0; x < img_w; x++)
-            {
-              SDL_GetRGB(getpixel(surf, x, y), surf->format, &r, &g, &b);
-              fprintf(fi, "%02x", (plane == 0 ? r : (plane == 1 ? g : b)));
+      for (x = 0; x < img_w; x++)
+      {
+        SDL_GetRGB(getpixel(surf, x, y), surf->format, &r, &g, &b);
+        fprintf(fi, "%02x", (plane == 0 ? r : (plane == 1 ? g : b)));
 
-              cur_line_len++;
-              if (cur_line_len >= 30)
-                {
-                  fprintf(fi, "\n");
-                  cur_line_len = 0;
-                }
-            }
+        cur_line_len++;
+        if (cur_line_len >= 30)
+        {
+          fprintf(fi, "\n");
+          cur_line_len = 0;
         }
+      }
     }
+  }
 
   fprintf(fi, "\n");
   fprintf(fi, "grestore\n");
@@ -288,74 +285,74 @@ int do_ps_save(FILE * fi,
   fprintf(fi, "%%%%EOF\n");
 
   if (!is_pipe)
-    {
-      fclose(fi);
-      return 1;
-    }
+  {
+    fclose(fi);
+    return 1;
+  }
   else
-    {
-      pid_t child_pid, w;
-      int status;
+  {
+    pid_t child_pid, w;
+    int status;
 
 #ifdef __APPLE__
-      /* macOS does not always reset errno so Tux Paint thinks print never
-       * succeeds - let's reset before calling pclose() on macOS */
-      errno = 0;
+    /* macOS does not always reset errno so Tux Paint thinks print never
+     * succeeds - let's reset before calling pclose() on macOS */
+    errno = 0;
 #endif
 
-      child_pid = pclose(fi);
+    child_pid = pclose(fi);
 
 #ifdef DEBUG
-      printf("pclose returned %d\n", child_pid);
-      fflush(stdout);
-      printf("errno = %d\n", errno);
-      fflush(stdout);
+    printf("pclose returned %d\n", child_pid);
+    fflush(stdout);
+    printf("errno = %d\n", errno);
+    fflush(stdout);
 #endif
 
-      if (child_pid < 0 || (errno != 0 && errno != EAGAIN))
-        {                       /* FIXME: This right? */
-          return 0;
-        }
-      else if (child_pid == 0)
-        {
-          return 1;
-        }
-
-      do
-        {
-          w = waitpid(child_pid, &status, 0);
-
-#ifdef DEBUG
-          if (w == -1)
-            {
-              perror("waitpid");
-              exit(EXIT_FAILURE);
-            }
-          if (WIFEXITED(status))
-            {
-              printf("exited, status=%d\n", WEXITSTATUS(status));
-            }
-          else if (WIFSIGNALED(status))
-            {
-              printf("killed by signal %d\n", WTERMSIG(status));
-            }
-          else if (WIFSTOPPED(status))
-            {
-              printf("stopped by signal %d\n", WSTOPSIG(status));
-            }
-          else if (WIFCONTINUED(status))
-            {
-              printf("continued\n");
-            }
-#endif
-        }
-      while (w != -1 && !WIFEXITED(status) && !WIFSIGNALED(status));
-
-      if (WIFEXITED(status) && WEXITSTATUS(status) != 0)        /* Not happy exit */
-        return 0;
-
+    if (child_pid < 0 || (errno != 0 && errno != EAGAIN))
+    {                           /* FIXME: This right? */
+      return 0;
+    }
+    else if (child_pid == 0)
+    {
       return 1;
     }
+
+    do
+    {
+      w = waitpid(child_pid, &status, 0);
+
+#ifdef DEBUG
+      if (w == -1)
+      {
+        perror("waitpid");
+        exit(EXIT_FAILURE);
+      }
+      if (WIFEXITED(status))
+      {
+        printf("exited, status=%d\n", WEXITSTATUS(status));
+      }
+      else if (WIFSIGNALED(status))
+      {
+        printf("killed by signal %d\n", WTERMSIG(status));
+      }
+      else if (WIFSTOPPED(status))
+      {
+        printf("stopped by signal %d\n", WSTOPSIG(status));
+      }
+      else if (WIFCONTINUED(status))
+      {
+        printf("continued\n");
+      }
+#endif
+    }
+    while (w != -1 && !WIFEXITED(status) && !WIFSIGNALED(status));
+
+    if (WIFEXITED(status) && WEXITSTATUS(status) != 0)  /* Not happy exit */
+      return 0;
+
+    return 1;
+  }
 }
 
 #endif
